@@ -245,6 +245,7 @@ main(int argc, char **argv)
 	int argCount = 1, i, j, logtype = BRISTOL_LOG_BRIGHTON;
 	pthread_t emgrThread, logthread;
 	char appname[65], *devname = defname;
+	int reqW = 0, reqH = 0, reqX = 0, reqY = 0;
 
 	signal(SIGINT, cleanupBristol);
 	signal(SIGPIPE, cleanupBristolQuietly);
@@ -489,6 +490,62 @@ main(int argc, char **argv)
 			continue;
 		}
 
+		if (((strcmp(argv[argCount], "-geometry") == 0)
+			|| (strcmp(argv[argCount], "-geom") == 0))
+			&& (argCount < argc))
+		{
+			int i = 0;
+			char *yp;
+
+			argCount++;
+
+			/*
+			 * We want WxH+x+y
+			 * If we only have +-x+-y then don't apply w.h
+			 * If we have H+x+y then use H as a width and use it to scale
+			 * the image to a given bitmap. If we have W+H then automatically
+			 * set -ar.
+			 */
+
+			/*
+			 * Start looking for width
+			 */
+			while ((argv[argCount][i] != '-')
+				&& (argv[argCount][i] != '+')
+				&& (argv[argCount][i] != '\0')
+				&& (argv[argCount][i] != 'x'))
+			{
+				reqW = reqW * 10 + argv[argCount][i] - 48;
+				i++;
+			}
+
+			if (argv[argCount][i] == 'x')
+				i++;
+
+			/* Look for height */
+			while ((argv[argCount][i] != '-')
+				&& (argv[argCount][i] != '+')
+				&& (argv[argCount][i] != '\0')
+				&& (argv[argCount][i] != 'x'))
+			{
+				reqH = reqH * 10 + argv[argCount][i] - 48;
+				i++;
+			}
+
+			/* Take an X if it is available */
+			if (argv[argCount][i] != '\0')
+				reqX = atoi(&argv[argCount][i]);
+			if ((reqX == 0) && (argv[argCount][i] == '-'))
+				reqX = -1;
+
+			if ((yp = strchr(&argv[argCount][++i], '+')))
+				reqY = atoi(yp);
+			else if ((yp = strchr(&argv[argCount][++i], '-')))
+				reqY = atoi(yp);
+			if ((reqY == 0) && (argv[argCount][i] == '-'))
+				reqY = -1;
+		}
+
 		if (strcmp(argv[argCount], "-iconify") == 0)
 			bwflags &= ~BRIGHTON_POST_WINDOW;
 
@@ -1350,6 +1407,30 @@ main(int argc, char **argv)
 	synthesisers[global.synths->synthtype]->flags ^= BRIGHTON_WINDOW;
 #endif
 
+	if (reqW)
+	{
+		/* Width is a scaler */
+		float scale;
+
+		scale = ((float) reqW) / synthesisers[global.synths->synthtype]->width;
+
+		/* Have been given some width value */
+		if (reqH)
+		{
+			nar = 1;
+			synthesisers[global.synths->synthtype]->width = reqW;
+			synthesisers[global.synths->synthtype]->height = reqH;
+		} else {
+			synthesisers[global.synths->synthtype]->width = reqW;
+			synthesisers[global.synths->synthtype]->height *= scale;
+		}
+
+		if ((scale > 1.1) || (scale < 0.9))
+			aliastype = 5;
+		else
+			aliastype = 0;
+	}
+
 	{
 		char statfile[1024];
 		struct stat statres;
@@ -1462,7 +1543,7 @@ main(int argc, char **argv)
 	synthesisers[global.synths->synthtype]->name = appname;
 	if ((global.synths->win =
 		brightonInterface(synthesisers[global.synths->synthtype],
-			quality, library, aliastype, antialias, gs)) == NULL)
+			quality, library, aliastype, antialias, gs, reqX, reqY)) == NULL)
 	{
 		printf("could not create window\n");
 		exit(-1);
@@ -1570,7 +1651,8 @@ main(int argc, char **argv)
 	brightonReadConfiguration(global.synths->win,
 		synthesisers[global.synths->synthtype],
 		global.synths->midichannel,
-		synthesisers[global.synths->synthtype]->name);
+		synthesisers[global.synths->synthtype]->name,
+		NULL);
 
 	if (bristolMidiSendMsg(global.controlfd, global.synths->sid,
 		BRISTOL_ACTIVE_SENSE, 0, activeSense == 0? 0:16383) != 0)
