@@ -748,7 +748,7 @@ brightonApp arp2600App = {
 	arp2600MidiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	886, 500,
+	886, 500, 0, 0,
 	1, /* Panels */
 	{
 		{
