@@ -275,7 +275,7 @@ brightonApp axxeApp = {
 	midiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	600, 350,
+	600, 350, 0, 0,
 	4, /* Panels */
 	{
 		{
