@@ -376,7 +376,7 @@ brightonApp bme700App = {
 	midiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	800, 417,
+	800, 417, 0, 0,
 	7,
 	{
 		{
