@@ -588,7 +588,7 @@ brightonApp bmApp = {
 	bmMidiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	820, 410,
+	820, 410, 0, 0,
 	8,
 	{
 		{
