@@ -750,7 +750,7 @@ brightonApp cs80App = {
 	midiCallback,
 	cs80DestroySynth,
 	{8, 150, 8, 2, 5, 520, 0, 0},
-	940, 435,
+	940, 435, 0, 0,
 	11,
 	{
 		{
