@@ -360,14 +360,14 @@ int channel, char *param)
  */
 void
 brightonReadConfiguration(brightonWindow *bwin, brightonApp *app,
-int channel, char *synth)
+int channel, char *synth, char *filename)
 {
 	FILE *fd;
 	char path[1024];
 	char param[256];
 	int kmnotdone = 1, i;
 
-	printf("Read Configuration: %s\n", synth);
+	printf("Read Configuration: %s %s\n", synth, filename);
 
 	/*
 	 * The build the value mapping table
@@ -378,11 +378,21 @@ int channel, char *synth)
 	 * We are going to look for a cached file. If we cannot find it then we
 	 * will look for a factory version
 	 */
-	sprintf(path, "%s/memory/profiles/%s", getBristolCache(synth), synth);
+	if (filename == NULL)
+		sprintf(path, "%s/memory/profiles/%s", getBristolCache(synth), synth);
+	else
+		sprintf(path, "%s", filename);
 
 	if ((fd = fopen(path, "r")) == NULL)
 	{
+		/*
+		 * If we could not read an explicit profile then return.
+		 */
+		if (filename != NULL)
+			return;
+
 		sprintf(path, "%s/memory/profiles/%s", getenv("BRISTOL"), synth);
+
 		if ((fd = fopen(path, "r")) == NULL)
 		{
 			/*
@@ -548,7 +558,8 @@ int channel, char *synth)
  * them out to a file with the controller ID.
  */
 void
-brightonWriteConfiguration(brightonWindow *bwin, char *synth, int channel)
+brightonWriteConfiguration(brightonWindow *bwin, char *synth, int channel,
+char *filename)
 {
 	int j, i, fd, null;
 	char path[256];
@@ -557,10 +568,20 @@ brightonWriteConfiguration(brightonWindow *bwin, char *synth, int channel)
 	if (global.synths->flags & REQ_MIDI_DEBUG)
 		printf("Write CC Configuration file: %s\n", synth);
 
-	sprintf(path, "%s/memory/profiles/%s", getBristolCache(synth), synth);
+	if (filename == NULL)
+		sprintf(path, "%s/memory/profiles/%s", getBristolCache(synth), synth);
+	else
+		sprintf(path, "%s", filename);
 
 	if ((fd = open(path, O_WRONLY|O_TRUNC|O_CREAT, 0644)) < 0)
 	{
+		/*
+		 * If we were requested to write a specific file but couldn't (implies
+		 * permissions or path?) then return.
+		 */
+		if (filename != NULL)
+			return;
+
 		sprintf(path, "%s/memory/profiles/%s", getenv("BRISTOL"), synth);
 		/*
 		 * We are unlikey to have write permissions on the factory set, however
