@@ -283,7 +283,7 @@ brightonApp explorerApp = {
 	midiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	817, 472,
+	817, 472, 0, 0,
 	7, /* one panel only */
 	{
 		{
