@@ -516,7 +516,7 @@ brightonApp granularApp = {
 	midiCallback,
 	destroySynth,
 	{16, 0, 2, 2, 5, 520, 0, 0},
-	740, 400,
+	740, 400, 0, 0,
 	4, /* panel count */
 	{
 		{
