@@ -621,7 +621,7 @@ brightonApp hammondB3App = {
 	hammondB3MidiCallback,
 	hammondB3destroy,
 	{-1, 0, 2, 1, 5, 520, 0, 0},
-	765, 400,
+	765, 400, 0, 0,
 	15,
 	{
 		{
