@@ -316,7 +316,7 @@ brightonApp junoApp = {
 	midiCallback,
 	destroySynth,
 	{6, 100, 1, 2, 5, 520, 0, 0},
-	820, 250,
+	820, 250, 0, 0,
 	5, /* one panel only */
 	{
 		{
