@@ -724,7 +724,7 @@ brightonApp jupiterApp = {
 	midiCallback,
 	jupiterDestroySynth,
 	{8, 50, 32, 2, 5, 520, 0, 0},
-	900, 350,
+	900, 350, 0, 0,
 	5,
 	{
 		{
