@@ -406,7 +406,7 @@ brightonApp ms20App = {
 	0,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	865, 440,
+	865, 440, 0, 0,
 	5, /* one panel only */
 	{
 		{
