@@ -204,7 +204,7 @@ brightonApp masterApp = {
 	masterMidiCallback,
 	destroySynth,
 	{8, 0, 2, 2, 5, 520, 0, 0},
-	1000, 300,
+	1000, 300, 0, 0,
 	8,
 	{
 		{
