@@ -386,7 +386,7 @@ brightonApp memMoogApp = {
 	MMmidiCallback,
 	destroySynth,
 	{6, 100, 4, 2, 5, 520, 0, 0},
-	817, 310,
+	817, 310, 0, 0,
 	8, /* Panels */
 	{
 		{
