@@ -238,7 +238,7 @@ brightonApp miniApp = {
 	midiCallback,
 	destroySynth, /* 0, */
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	700, 400, /*/367, */
+	700, 400, 0, 0, /*/367, */
 	9,
 	{
 		{
