@@ -312,7 +312,7 @@ brightonApp obxApp = {
 	obxMidiCallback,
 	destroySynth,
 	{5, 100, 2, 2, 5, 520, 0, 0},
-	770, 350,
+	770, 350, 0, 0,
 	6, /* Panel count */
 	{
 		{
