@@ -406,7 +406,7 @@ brightonApp obxaApp = {
 	obxaMidiCallback,
 	destroySynth,
 	{10, 100, 2, 2, 5, 520, 55, 0},
-	770, 350,
+	770, 350, 0, 0,
 	6, /* panel count */
 	{
 		{
