@@ -504,7 +504,7 @@ brightonApp odysseyApp = {
 	midiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	600, 450,
+	600, 450, 0, 0,
 	6, /* Panels */
 	{
 		{
