@@ -263,7 +263,7 @@ brightonApp polyApp = {
 	pmidiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	750, 400,
+	750, 400, 0, 0,
 	7,
 	{
 		{
