@@ -240,7 +240,7 @@ brightonApp poly6App = {
 	midiCallback,
 	destroySynth,
 	{6, 100, 2, 2, 5, 520, 0, 0},
-	800, 240,
+	800, 240, 0, 0,
 	5, /* one panel only */
 	{
 		{
