@@ -317,7 +317,7 @@ brightonApp pro1App = {
 	midiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	895, 500,
+	895, 500, 0, 0,
 	5,
 	{
 		{
