@@ -252,7 +252,7 @@ brightonApp prophetApp = {
 	midiCallback,
 	destroySynth,
 	{5, 100, 2, 2, 5, 520, 0, 0},
-	745, 310,
+	745, 310, 0, 0,
 	7,
 	{
 		{
