@@ -387,7 +387,7 @@ brightonApp pro10App = {
 	midiCallback,
 	pro10destroy,
 	{10, 100, 2, 2, 5, 520, 0, 0},
-	745, 410,
+	745, 410, 0, 0,
 	7,
 	{
 		{
