@@ -243,7 +243,7 @@ brightonApp prophet52App = {
 	midiCallback,
 	destroySynth,
 	{5, 100, 2, 2, 5, 520, 0, 0},
-	753, 310,
+	753, 310, 0, 0,
 	6, /* one panel only */
 	{
 		{
