@@ -204,7 +204,7 @@ brightonApp realisticApp = {
 	0,
 	/* This looks like 32 voices however the synth is single voice in preops */
 	{32, 100, 2, 2, 5, 520, 0, 0},
-	600, 320,
+	600, 320, 0, 0,
 	2, /* panel count */
 	{
 		{
