@@ -434,7 +434,7 @@ brightonApp roadrunnerApp = {
 	midiCallback,
 	destroySynth,
 	{-1, 0, 2, 2, 5, 520, 0, 0},
-	660, 138,
+	660, 138, 0, 0,
 	7, /* panel count */
 	{
 		{
