@@ -37,7 +37,6 @@
 
 extern guimain global;
 
-extern void brightonWriteConfiguration(brightonWindow *, char *, int);
 extern void brightonMidiInput(bristolMidiMsg *, guimain *);
 
 struct sockaddr address;
@@ -409,7 +408,7 @@ int
 bristolMemoryExport(int source, char *dst, char *algo)
 {
 	char srcpath[1024];
-	char seqpath[1024];
+	char fpath[1024];
 
 	snprintf(srcpath, 1024, "%s/memory/%s/%s%i.mem",
 		getBristolCache(algo), algo, algo, source);
@@ -419,8 +418,15 @@ bristolMemoryExport(int source, char *dst, char *algo)
 
 	snprintf(srcpath, 1024, "%s/memory/%s/%s%i.seq",
 		getBristolCache(algo), algo, algo, source);
-	snprintf(seqpath, 1024, "%s.seq", dst);
-	bsmCopy(srcpath, seqpath);
+	snprintf(fpath, 1024, "%s.seq", dst);
+	bsmCopy(srcpath, fpath);
+
+	snprintf(fpath, 1024, "%s.prof", dst);
+	brightonWriteConfiguration(
+		global.synths->win,
+		algo,
+		global.synths->midichannel,
+		fpath);
 
 	return(0);
 }
@@ -429,7 +435,7 @@ int
 bristolMemoryImport(int dest, char *src, char *algo)
 {
 	char dstpath[1024];
-	char seqpath[1024];
+	char fpath[1024];
 
 	snprintf(dstpath, 1024, "%s/memory/%s/%s%i.mem",
 		getBristolCache(algo), algo, algo, dest);
@@ -437,10 +443,19 @@ bristolMemoryImport(int dest, char *src, char *algo)
 
 	printf("Import %s to %s\n", src, dstpath);
 
-	snprintf(seqpath, 1024, "%s.seq", src);
+	snprintf(fpath, 1024, "%s.seq", src);
 	snprintf(dstpath, 1024, "%s/memory/%s/%s%i.seq",
 		getBristolCache(algo), algo, algo, dest);
-	bsmCopy(seqpath, dstpath);
+	bsmCopy(fpath, dstpath);
+
+	snprintf(fpath, 1024, "%s.prof", src);
+	brightonReadConfiguration(
+		global.synths->win,
+		global.synths->win->template,
+		global.synths->midichannel,
+		algo,
+		fpath);
+	//brightonReadConfiguration(synth->win, algo, synth->midichannel, fpath);
 
 	return(0);
 }
@@ -823,9 +838,10 @@ saveMemory(guiSynth *synth, char *algo, char *name, int location, int skip)
 		printf("mem write failed 2\n");
 
 	/*
-	 * Save profile with controller registrations.
+	 * Save global profile with controller registrations. No filename is given
+	 * so that the called code will take the cache copy.
 	 */
-	brightonWriteConfiguration(synth->win, algo, synth->midichannel);
+	brightonWriteConfiguration(synth->win, algo, synth->midichannel, NULL);
 
 	close(fd);
 
