@@ -395,7 +395,7 @@ brightonApp sAksApp = {
 	0,
 	destroySynth,
 	{1, 0, 2, 2, 5, 520, 0, 0},
-	700, 500,
+	700, 500, 0, 0,
 	1, /* Panels */
 	{
 		{
