@@ -619,7 +619,7 @@ brightonApp sidApp = {
 	midiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	1000, 330,
+	1000, 330, 0, 0,
 	3, /* panels */
 	{
 		{
