@@ -899,7 +899,7 @@ brightonApp sid2App = {
 	midiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	1000, 450,
+	1000, 450, 0, 0,
 	3, /* panels */
 	{
 		{
