@@ -399,7 +399,7 @@ brightonApp sonic6App = {
 	midiCallback,
 	destroySynth,
 	{1, 100, 2, 2, 5, 520, 0, 0},
-	900, 520,
+	900, 520, 0, 0,
 	3,
 	{
 		{
