@@ -498,7 +498,7 @@ brightonApp stratusApp = {
 	stratusMidiCallback,
 	destroySynth,
 	{5, 100, 1, 2, 5, 520, 0, 0},
-	840, 385,
+	840, 385, 0, 0,
 	7, /* Panel count */
 	{
 		{
