@@ -408,7 +408,7 @@ brightonApp trilogyApp = {
 	stratusMidiCallback,
 	destroySynth,
 	{5, 100, 1, 2, 5, 520, 0, 0},
-	900, 385,
+	900, 385, 0, 0,
 	7, /* Panel count */
 	{
 		{
