@@ -444,7 +444,7 @@ brightonApp voxM2App = {
 	midiCallback,
 	destroySynth,
 	{-1, 0, 2, 2, 5, 520, 0, 0},
-	520, 310,
+	520, 310, 0, 0,
 	9, /* panel count */
 	{
 		{
