@@ -32,6 +32,10 @@
 #include "bristolmidi.h"
 #include "palette.h"
 
+#ifdef BRISTOL_PA
+extern int bristolPulseInterface();
+#endif
+
 #ifdef _BRISTOL_JACK
 extern int bristolJackInterface();
 #endif
@@ -95,6 +99,36 @@ audioThread(audioMain *audiomain)
 	 * not the same with jack, we register a callback and let it do the work
 	 * for us.
 	 */
+
+	if (audiomain->flags & BRISTOL_PULSE)
+	{
+#ifdef BRISTOL_PA
+		while (audiomain->atReq != BRISTOL_REQSTOP)
+		{
+			/*
+			 * This will not return except when problem arise with the jack
+			 * interface. The returning code will decide whether to flag a
+			 * reqstop to exit
+			 */
+			bristolPulseInterface(audiomain);
+
+			audiomain->atStatus = BRISTOL_EXIT;
+
+			printf("pulse audio interface returned\n");
+
+			_exit(0);
+		}
+
+		return;
+#else
+		printf("pulse requested but not compiled with engine\n");
+
+		audiomain->atStatus = BRISTOL_EXIT;
+
+		_exit(0);
+#endif /* BRISTOL_PA */
+	}
+
 	if (audiomain->flags & BRISTOL_JACK)
 	{
 #ifdef _BRISTOL_JACK
@@ -116,7 +150,7 @@ audioThread(audioMain *audiomain)
 
 		return;
 #else
-		printf("Jack requested but not compiled with engine\n");
+		printf("jack requested but not compiled with engine\n");
 
 		audiomain->atStatus = BRISTOL_EXIT;
 
@@ -1026,7 +1060,7 @@ initPalette(audioMain *audiomain, bristolOP *palette[])
 	for (i = 0; i < BRISTOL_SYNTHCOUNT; i++)
 	{
 		if (bristolPalette[i].initialise == NULL)
-			break;
+			continue;
 
 		palette[i] = bristolPalette[i].initialise
 			(&palette[i], i, audiomain->samplerate,
