@@ -37,7 +37,7 @@
 #include "bristolmidi.h"
 #include "bristolblo.h"
 
-static const char *defaultcdev = "localhost";
+static char *defaultcdev = "localhost";
 
 extern int dupfd;
 void inthandler();
@@ -59,6 +59,10 @@ static char sessionfile[1024];
 
 char *outputfile = NULL;
 
+#ifdef BRISTOL_PA
+extern int bristolPulseInterface();
+#endif
+
 #ifdef _BRISTOL_JACK
 extern int bristolJackInterface();
 #ifdef _BRISTOL_JACK_SESSION
@@ -109,9 +113,8 @@ buildCmdLine(audioMain *audiomain, int argc, char **argv)
 
 	audiomain->cmdline = bristolmalloc0(size + 1);
 
-	sprintf(audiomain->cmdline, "startBristol");
-
 	printf("%s", argv[0]);
+	sprintf(audiomain->cmdline, "startBristol");
 
 	for (i = 1; i < argc; i++)
 	{
@@ -127,6 +130,8 @@ buildCmdLine(audioMain *audiomain, int argc, char **argv)
 				haveport = 1;
 				i++;
 				printf(" %s", argv[i]);
+	//			sprintf(audiomain->cmdline,
+	//				"%s -port %s", audiomain->cmdline, argv[i]);
 				continue;
 			}
 			haveport = 1;
@@ -136,6 +141,7 @@ buildCmdLine(audioMain *audiomain, int argc, char **argv)
 		if ((strcmp(argv[i], "-jsmuuid") == 0)
 			|| (strcmp(argv[i], "-rate") == 0)
 			|| (strcmp(argv[i], "-count") == 0)
+			|| (strcmp(argv[i], "-port") == 0)
 			|| (strcmp(argv[i], "-jsmfile") == 0))
 		{
 			i++;
@@ -144,7 +150,7 @@ buildCmdLine(audioMain *audiomain, int argc, char **argv)
 		}
 		sprintf(audiomain->cmdline, "%s %s", audiomain->cmdline, argv[i]);
 	}
-	printf("\n%s\n", audiomain->cmdline);
+	printf("\njsm will use '%s'\n", audiomain->cmdline);
 }
 
 /*
@@ -189,14 +195,24 @@ main(int argc, char **argv)
 	/* Default the devicename, can be overridden later */
 	audiomain.audiodev = argv[0];
 #else
+#if (BRISTOL_HAS_ALSA == 1)
 	audiomain.flags = BRISTOL_ALSA|BRISTOL_MIDI_WAIT;
+#else
+	/* So default to OSS */
+	audiomain.flags = BRISTOL_OSS;
+#endif
 #endif
 
 #ifdef BRISTOL_JACK_DEFAULT_MIDI
 	audiomain.flags |= BRISTOL_MIDI_JACK;
 	/* Default the devicename, can be overridden later */
 #else
+#if (BRISTOL_HAS_ALSA == 1)
 	audiomain.flags |= BRISTOL_MIDI_SEQ;
+#else
+	/* So default to OSS */
+	audiomain.flags = BRISTOL_MIDI_OSS;
+#endif
 #endif
 
 	audiomain.samplerate = 44100; /* This is a default, gets overwritten */
@@ -322,7 +338,8 @@ main(int argc, char **argv)
 				 * provide midi messaging, however it is possible to have jack
 				 * audio drivers with ALSA rawmidi interfacing, for example.
 				 */
-				if (strcmp(argv[argCount], "none") == 0)
+				if ((strcmp(argv[argCount], "none") == 0)
+					|| (strcmp(argv[argCount], "dummy") == 0))
 					audiomain.flags &= ~BRISTOL_MIDIMASK;
 				else if (strcmp(argv[argCount], "oss") == 0)
 					audiomain.flags = (audiomain.flags & ~BRISTOL_MIDIMASK)
@@ -382,14 +399,24 @@ main(int argc, char **argv)
 					audiomain.flags
 						= (audiomain.flags & ~BRISTOL_AUDIOMASK)|BRISTOL_ALSA;
 					audiomain.audiodev = NULL;
-				} else if (strcmp(argv[argCount], "slab") == 0)
+				} else if (strcmp(argv[argCount], "slab") == 0) {
 					audiomain.flags
 						= (audiomain.flags & ~BRISTOL_AUDIOMASK)|BRISTOL_SLAB;
-				else if (strcmp(argv[argCount], "jack") == 0) {
+				} else if (strcmp(argv[argCount], "jack") == 0) {
 					audiomain.flags
 						= (audiomain.flags & ~BRISTOL_AUDIOMASK)|BRISTOL_JACK;
 					/* Default the devicename, can be overridden later */
 					audiomain.audiodev = argv[0];
+				} else if (strcmp(argv[argCount], "pulsethread") == 0) {
+					audiomain.flags
+						= (audiomain.flags & ~BRISTOL_AUDIOMASK)|BRISTOL_PULSE;
+					/* Default the devicename, can be overridden later */
+					audiomain.audiodev = argv[0];
+				} else if (strcmp(argv[argCount], "pulse") == 0) {
+					audiomain.flags
+						= (audiomain.flags &~BRISTOL_AUDIOMASK)|BRISTOL_PULSE_S;
+					/* Default the devicename, can be overridden later */
+					audiomain.audiodev = argv[0];
 				} else if (strcmp(argv[argCount], "dummy") == 0)
 					audiomain.flags = (audiomain.flags &
 						~(BRISTOL_AUDIOMASK|BRISTOL_MIDIMASK|BRISTOL_MIDI_ALSA))
@@ -472,7 +499,6 @@ main(int argc, char **argv)
 		{
 			snprintf(audiomain.jackUUID, BRISTOL_JACK_UUID_SIZE, "%s",
 				argv[++argCount]);
-			++argCount;
 		}
 
 		if (((strcmp(argv[argCount], "-jsmfile") == 0) ||
@@ -857,6 +883,14 @@ main(int argc, char **argv)
 #endif
 	}
 
+	if (audiomain.flags & BRISTOL_PULSE)
+	{
+		audiomain.atReq = BRISTOL_REQSTOP;
+#ifdef BRISTOL_PA
+		bristolPulseInterface(NULL);
+#endif
+	}
+
 	/* Unregister the jack interface */
 	if ((jh >= 0) && (audiomain.flags & BRISTOL_MIDI_JACK)
 		&& (audiomain.flags & BRISTOL_JACK_DUAL))
