@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -313,8 +312,10 @@ bristolVoice *voice, register float *startbuf)
 	 * See if we need to merge the lower manual
 	 */
 	if (tmpbuf4)
+	{
 		bufmerge(tmpbuf4, 1.0, baudio->leftbuf, 1.0, samplecount);
-	bristolbzero(tmpbuf4, audiomain->segmentsize);
+		bristolbzero(tmpbuf4, audiomain->segmentsize);
+	}
 
 	if (baudio->mixflags & HAMMOND_VIBRA)
 	{
@@ -587,7 +588,7 @@ printf("initialising one hammond sound\n");
 		bristolmalloc0(sizeof(bristolOP *) * baudio->soundCount);
 
 	if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG3)
-		printf("malloced sound: %x\n", (size_t) baudio->sound);
+		printf("malloced sound: %p\n", baudio->sound);
 
 	/*
 	 * Hammond oscillator
@@ -647,6 +648,7 @@ printf("initialising one hammond sound\n");
 		tmpbuf1 = (float *) bristolmalloc0(audiomain->segmentsize);
 		tmpbuf2 = (float *) bristolmalloc0(audiomain->segmentsize);
 	}
+
 	return(0);
 }
 
@@ -696,6 +698,7 @@ printf("initialising one hammond second manual\n");
 		tmpbuf3 = (float *) bristolmalloc0(audiomain->segmentsize);
 		tmpbuf4 = (float *) bristolmalloc0(audiomain->segmentsize);
 	}
+
 	return(0);
 }
 
