@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -196,6 +195,17 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 		local->cstate = STATE_ATTACK;
 		/* At keyon we really need to stuff a starting value */
 		cgain = param->param[0].float_val;
+
+		if ((~voice->flags & BRISTOL_KEYREON) &&
+			(voice->offset > 0) && (voice->offset < count))
+		{
+			if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+				printf("envelope trigger offset %i frames\n", voice->offset);
+
+			memset(ob, 0.0f, voice->offset * sizeof(float));
+			ob += voice->offset;
+			count -= voice->offset;
+		}
 	}
 
 	/*
