@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -133,6 +132,13 @@ static int reset(bristolOP *operator, bristolOPParams *param)
 	printf("hammondreset(%x, %x)\n", operator, param);
 #endif
 
+	if (param->param[0].mem) bristolfree(param->param[0].mem);
+	if (param->param[1].mem) bristolfree(param->param[1].mem);
+	if (param->param[2].mem) bristolfree(param->param[2].mem);
+	if (param->param[3].mem) bristolfree(param->param[3].mem);
+	if (param->param[4].mem) bristolfree(param->param[4].mem);
+	if (param->param[5].mem) bristolfree(param->param[5].mem);
+
 	/*
 	 * For the hammond we want to put in a few local memory variables for this
 	 * operator specifically.
