@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -28,6 +27,9 @@
 
 extern bristolMidiHandler bristolMidiRoutines;
 
+int rbMidiNoteOn(audioMain *, bristolMidiMsg *);
+int rbMidiNoteOff(audioMain *, bristolMidiMsg *);
+
 /*
  * Monophonic voice logic
  */
@@ -35,74 +37,107 @@ static int
 doMNL(bristolVoice *voice, int key, int velocity, int action)
 {
 	Baudio *baudio = voice->baudio;
-	int mn = -1, ncount = 0;
+	int mn = -1;
 
 	if (voice->baudio == NULL)
 		return(0);
 
+	if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+		printf("monoPhonic Note Logic (%i %i)\n", key, action);
+
 	if (action == BRISTOL_ALL_NOTES_OFF)
 	{
-		for (mn = 0; mn < 128; mn++)
-			baudio->notemap.key[mn] = 0;
-			
 		/*
 		 * Clear out our table
 		 */
+		for (mn = 0; mn < 128; mn++)
+			baudio->notemap.key[mn] = 0;
+		baudio->notemap.count = 0;
+		voice->flags &= ~BRISTOL_KEYSUSTAIN;
 		return(0);
 	}
 
-	if (voice->baudio->notemap.flags == 0)
+	if ((voice->baudio->notemap.flags & (BRISTOL_MNL_LNP|BRISTOL_MNL_HNP)) == 0)
 		return(-1);
 
+	/* We never use this, the key actually stays on all the time anyway */
+	voice->flags &= ~BRISTOL_KEYSUSTAIN;
+
 	baudio->notemap.key[key] = action * key;
 	baudio->notemap.velocity[key] = velocity;
 
-	/* printf("notemap.key[%i](%i)\n", key, action); */
-	if (action)
-		baudio->notemap.count++;
-	else
-		baudio->notemap.count--;
+	baudio->notemap.low = 127;
+	baudio->notemap.high = 0;
+	baudio->notemap.count = 0;
 
-	if (baudio->notemap.flags & BRISTOL_MNL_LNP)
+	for (mn = 0; mn < 128; mn++)
 	{
-		for (mn = 0; mn < 128; mn++)
-			if (baudio->notemap.key[mn] != 0)
-			{
-				ncount++;
-				break;
-			}
-
-		if ((ncount == 0) || (mn >= 128))
+		if (baudio->notemap.key[mn] != 0)
 		{
-			voice->flags |= BRISTOL_KEYOFF;
-			return(-1);
-		}
-		if (key > mn)
-		{
-			voice->flags &= ~(BRISTOL_KEYON|BRISTOL_KEYREON);
-			return(-1);
-		}
-	} else {
-		for (mn = 127; mn >= 0; mn--)
-			if (baudio->notemap.key[mn] != 0)
-			{
-				ncount++;
-				break;
-			}
-
-		if ((ncount == 0) || (mn < 0))
-		{
-			voice->flags |= BRISTOL_KEYOFF;
-			return(-1);
-		}
-		if (key < mn)
-		{
-			voice->flags &= ~(BRISTOL_KEYON|BRISTOL_KEYREON);
-			return(-1);
+			baudio->notemap.count++;
+			if (mn < baudio->notemap.low) baudio->notemap.low = mn;
+			if (mn > baudio->notemap.high) baudio->notemap.high = mn;
 		}
 	}
 
-/* printf("(%i) C: %i, mn %i, key %i\n", baudio->notemap.count, c, mn, key); */
+	if (baudio->notemap.flags & BRISTOL_MNL_LNP)
+	{
+		if ((baudio->notemap.extreme <= 0)
+			|| (baudio->notemap.extreme < baudio->notemap.high))
+			baudio->notemap.extreme = baudio->notemap.high;
+
+		if ((baudio->notemap.count == 0)
+			&& (baudio->contcontroller[BRISTOL_CC_HOLD1] < 0.5))
+		{
+			voice->flags |= BRISTOL_KEYOFF;
+			return(-1);
+		}
+
+		if (key > baudio->notemap.low)
+		{
+			/*
+			 * There is an issue here: If I get a note on then a note off in 
+			 * the same sample period then the first note correctly sets the
+			 * keyon flags, the second one incorrectly clears them. Now I am 
+			 * in the audio thread here so there is nothing to signal to the
+			 * audio process to find out if it has seen my flags yet.
+			 * The question is, why would I clear these flags on note off that
+			 * is not in my key range?
+			voice->flags &= ~(BRISTOL_KEYON|BRISTOL_KEYREON);
+			 */
+			return(-1);
+		}
+
+		mn = baudio->notemap.low;
+	} else {
+		if ((baudio->notemap.extreme <= 0)
+			|| (baudio->notemap.extreme > baudio->notemap.low))
+			baudio->notemap.extreme = baudio->notemap.low;
+
+		if ((baudio->notemap.count == 0)
+			&& (baudio->contcontroller[BRISTOL_CC_HOLD1] < 0.5))
+		{
+			voice->flags |= BRISTOL_KEYOFF;
+			return(-1);
+		}
+
+		if (key < baudio->notemap.high)
+		{
+			/*
+			 * See above notes - this key should not do anything, at all
+			voice->flags &= ~(BRISTOL_KEYON|BRISTOL_KEYREON);
+			*/
+			return(-1);
+		}
+
+		mn = baudio->notemap.high;
+	}
+
+	if (baudio->notemap.count == 0)
+		mn = baudio->notemap.extreme;
+
+	if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG2)
+		printf("(%i) mn %i, key %i\n", baudio->notemap.count, mn, key);
 
 	mn += baudio->transpose;
 
@@ -112,6 +147,7 @@ doMNL(bristolVoice *voice, int key, int velocity, int action)
 		mn += 12;
 
 	voice->lastkey = voice->key.key;
+	voice->keyid = key;
 	voice->key.key = mn;
 
 	if (baudio->microtonalmap[mn].step > 0.0) {
@@ -199,32 +235,25 @@ printPlayList(audioMain *audiomain)
 {
 	bristolVoice *v = audiomain->playlist;
 
-	printf("printPlayList(%x, %x)\n",
-		(size_t) audiomain->playlist, (size_t) audiomain->playlast);
+	printf("printPlayList(%p, %p)\n", audiomain->playlist, audiomain->playlast);
 
 	while (v != NULL)
 	{
-		printf("%08x<-%08x->%08x: %i, %i, 0x%x\n",
-			(size_t) v->last,
-			(size_t) v,
-			(size_t) v->next,
-			v->index,
-			v->key.key,
-			v->flags
-		);
+		printf("%p<-%p->%p: %i, %i, 0x%x (%p)\n",
+			v->last, v, v->next, v->index, v->key.key, v->flags, v->baudio);
 
 		v = v->next;
 	}
 
 	if ((v = audiomain->newlist) != NULL)
 	{
-		printf("printNewlist(%x)\n", (size_t) v);
+		printf("printNewlist(%p)\n", v);
 		while (v != NULL)
 		{
-			printf("%08x<-%08x->%08x: %i, %i, 0x%x\n",
-				(size_t) v->last,
-				(size_t) v,
-				(size_t) v->next,
+			printf("%p<-%p->%p: %i, %i, 0x%x\n",
+				v->last,
+				v,
+				v->next,
 				v->index,
 				v->key.key,
 				v->flags
@@ -239,14 +268,12 @@ printPlayList(audioMain *audiomain)
  * There will be a lot of processing involved with MIDI note on/off management,
  * so it has been pulled into a separate file.
  */
-
 int
 midiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 {
-	bristolVoice *voice, *v;
 
-//printf("mNOff ");printPlayList(audiomain);
 	/*
+printf("midiNoteOff %i\n", msg->params.key.key);
 	 * If we get a note off, then find that note, and configure it for done. If
 	 * we have sustain active then just set the BRISTOL_SUSTAIN flag, the key
 	 * will be released when the sustain pedal is released.
@@ -258,13 +285,42 @@ midiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 	 * the long semaphore too. Hence we just request the long one as it covers
 	 * both needs.
 	 */
+#ifdef BRISTOL_SEMAPHORE
+	bristolVoice *voice, *v;
 	sem_wait(audiomain->sem_long);
+#else
+	if ((msg->params.key.flags & BRISTOL_KF_JACK)
+		&& (~audiomain->flags & BRISTOL_JACK_DUAL))
+	{
+		/*
+		 * The message came from Jack and we have selected a single registration
+		 * which means we are actually in the audio thread and can conginue with
+		 * the note on event.
+		 */
+	    if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG2)
+			printf("direct off call\n");
+		return(rbMidiNoteOff(audiomain, msg));
+	} if (jack_ringbuffer_write_space(audiomain->rb) >= sizeof(bristolMidiMsg))
+		jack_ringbuffer_write(audiomain->rb, (char *) msg,
+			sizeof(bristolMidiMsg));
+	else
+		printf("ringbuffer exhausted, note off event dropped\n");
+
+	return(0);
+}
+
+int
+rbMidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
+{
+	bristolVoice *voice, *v;
+#endif
 
 	voice = audiomain->playlist;
 
 	while (voice != NULL)
 	{
 		if ((voice->baudio == NULL)
+			|| ((voice->baudio->mixflags & (BRISTOL_HOLDDOWN|BRISTOL_REMOVE)))
 			|| (voice->baudio->mixflags & BRISTOL_KEYHOLD))
 		{
 			voice = voice->next;
@@ -280,7 +336,7 @@ midiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 		if ((voice->baudio->voicecount == 1)
 			&& ((voice->baudio->midichannel == msg->channel)
 				|| (voice->baudio->midichannel == BRISTOL_CHAN_OMNI))
-			&& (voice->baudio->notemap.flags != 0))
+			&& (voice->baudio->notemap.flags & (BRISTOL_MNL_LNP|BRISTOL_MNL_HNP)))
 		{
 			if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
 				bristolMidiPrint(msg);
@@ -297,7 +353,8 @@ midiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 			if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
 				bristolMidiPrint(msg);
 
-			if (voice->baudio->contcontroller[BRISTOL_CC_HOLD1] > 0)
+			if ((voice->baudio->contcontroller[BRISTOL_CC_HOLD1] > 0.5)
+				&& (voice->baudio->voicecount > 1))
 				voice->flags |= BRISTOL_KEYSUSTAIN;
 			else {
 				/*
@@ -369,8 +426,8 @@ midiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 			audiomain->freelist = voice;
 
 			if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG2)
-				printf("mt: off: removed %x/%i from newlist\n",
-					(size_t) voice, voice->keyid);
+				printf("mt: off: removed %p/%i from newlist\n",
+					voice, voice->keyid);
 
 			voice = v;
 			continue;
@@ -379,7 +436,9 @@ midiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 		voice = voice->next;
 	}
 
+#ifdef BRISTOL_SEMAPHORE
 	sem_post(audiomain->sem_long);
+#endif
 
 //printf("sl: ");printPlayList(audiomain);
 
@@ -394,15 +453,13 @@ midiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
  * for the ability to do layering.
  */
 int
-domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
+doMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 {
 	bristolVoice *voice, *last = NULL;
-	int lastkey, transposedkey, velocity;
+	int lastkey, transposedkey, velocity, offset;
 	float cFreq, dFreq, dTune = 0.0, cFreqmult = 0, mappedvelocity;
 	float cfreq, dfreq, cfreqmult;
 
-//printf("domidiNoteOn()\n");
-//printf("dmNOn: ");printPlayList(audiomain);
 	/*
 	 *	locals
 	 *	Keyid
@@ -413,7 +470,13 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 	 *	cFreqmult
 	 */
 	if (baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+	{
+		printf("doMidiNoteOn()\n");
 		bristolMidiPrint(msg);
+	}
+
+	if (baudio->midiflags & BRISTOL_MIDI_DEBUG2)
+		printPlayList(audiomain);
 
 	/*
 	 * We may have to do some foldback however steps of 12 would only work for
@@ -427,6 +490,9 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 	while (transposedkey < 0)
 		transposedkey += 12;
 
+	if ((offset = msg->offset) >= audiomain->samplecount)
+		offset = 0;
+
 	/*
 	 * Get some operational values ready for the voice structures.
 	 */
@@ -444,8 +510,14 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 	lastkey = baudio->lastkey;
 	baudio->lastkey = transposedkey;
 
+	/*
+	 * Have been requested to implement this feature as a part of Unison/Mono
+	 * mode of the diverse poly instruments. This should be possible with an
+	 * OR operation on the UNISON flag however this is currently handled in the
+	 * calling routine, not here.
+	 */
 	if ((baudio->voicecount == 1) && (baudio->lvoices != 0)
-		&& (baudio->notemap.flags != 0))
+		&& (baudio->notemap.flags & (BRISTOL_MNL_LNP|BRISTOL_MNL_HNP)))
 	{
 		if ((voice = audiomain->playlist) == NULL)
 			return(0);
@@ -516,7 +588,7 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 		dFreq *= dTune;
 		dfreq *= dTune;
 
-/*printf("tampering key by %1.4f, was %1.4f, is %1.4f\n", */
+/*printf("tempering key by %1.4f, was %1.4f, is %1.4f\n", */
 /*dTune, voice->baudio->ctab[transposedkey].step, voice->dFreq); */
 	}
 /* printf("		detune %f\n", baudio->detune); */
@@ -596,6 +668,7 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 		voice = audiomain->freelist;
 		voice->baudio = baudio;
 		voice->flags &= ~(BRISTOL_KEYON|BRISTOL_KEYREON);
+		voice->offset = offset;
 		voice->locals = baudio->locals;
 		voice->keyid = key;
 		voice->key.key = transposedkey;
@@ -610,7 +683,9 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 		voice->cfreqmult = cfreqmult;
 		voice->detune = dTune;
 
+#ifdef BRISTOL_SEMAPHORE
 		sem_wait(audiomain->sem_short);
+#endif
 
 		/*
 		 * Short piece of critical code
@@ -634,12 +709,16 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 
 //printf("ss: ");printPlayList(audiomain);
 
+#ifdef BRISTOL_SEMAPHORE
 		sem_post(audiomain->sem_short);
+#endif
 
 		return(0);
 	}
 
+#ifdef BRISTOL_SEMAPHORE
 	sem_wait(audiomain->sem_long);
+#endif
 
 	/*
 	 * We have the semaphore, search for the voice. There are several cases
@@ -717,6 +796,7 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 			voice->flags |= BRISTOL_KEYREON;
 
 		voice->keyid = key;
+		voice->offset = offset;
 		voice->key.key = transposedkey;
 		voice->key.velocity = velocity;
 		voice->velocity = mappedvelocity;
@@ -732,7 +812,9 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 		if (baudio->midiflags & BRISTOL_MIDI_DEBUG2)
 			printf("voicecount exceeded\n");
 
+#ifdef BRISTOL_SEMAPHORE
 	sem_post(audiomain->sem_long);
+#endif
 
 //printf("sl: ");printPlayList(audiomain);
 
@@ -742,8 +824,42 @@ domidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 int
 midiNoteOn(audioMain *audiomain, bristolMidiMsg *msg)
 {
+#ifdef BRISTOL_SEMAPHORE
 	Baudio *baudio = audiomain->audiolist;
+#else
 
+	/*
+	 * So, if this came from a jack handle, and we are single open then don't
+	 * dump this in the ringbuffer - call the handler directly
+	 *
+	 * The JDO flags is straightforward. We then need to check if 'from' is 
+	 * a jack handle.
+	 */
+	if ((msg->params.key.flags & BRISTOL_KF_JACK)
+		&& (~audiomain->flags & BRISTOL_JACK_DUAL))
+	{
+		/*
+		 * The message came from Jack and we have selected a single registration
+		 * which means we are actually in the audio thread and can conginue with
+		 * the note on event.
+		 */
+	    if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG2)
+			printf("direct on call\n");
+		return(rbMidiNoteOn(audiomain, msg));
+	} if (jack_ringbuffer_write_space(audiomain->rb) >= sizeof(bristolMidiMsg))
+		jack_ringbuffer_write(audiomain->rb, (char *) msg,
+			sizeof(bristolMidiMsg));
+	else
+		printf("ringbuffer exhausted, note on event dropped\n");
+
+	return(0);
+}
+
+int
+rbMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg)
+{
+	Baudio *baudio = audiomain->audiolist;
+#endif
 	/*
 	 * Hm, if we have already applied a velocity curve then this value that
 	 * was zero can now be something else, and that would lead to unexpected
@@ -759,6 +875,12 @@ midiNoteOn(audioMain *audiomain, bristolMidiMsg *msg)
 	 */
 	while (baudio != (Baudio *) NULL)
 	{
+		if (baudio->mixflags & (BRISTOL_HOLDDOWN|BRISTOL_REMOVE))
+		{
+			baudio = baudio->next;
+			continue;
+		}
+
 		if (((baudio->midichannel == msg->channel)
 				|| (baudio->midichannel == BRISTOL_CHAN_OMNI))
 			&& (baudio->lowkey <= msg->params.key.key)
@@ -810,14 +932,16 @@ midiNoteOn(audioMain *audiomain, bristolMidiMsg *msg)
 					 * code, it would be a lot cleaner. Overhead is not great
 					 * as its only for note on events. There is no reason why
 					 * the arpeggiator should not actually call the shots here
-					 * and request the domidiNoteOn() back to this code.
-printf("chord: %x %i %i\n", (size_t) baudio, baudio->arpeggio.c.max,
-baudio->arpeggio.c.notes[chordindex - 1].k);
+					 * and request the doMidiNoteOn() back to this code.
+						printf("chord: %x %i %i\n",
+							(size_t) baudio,
+							baudio->arpeggio.c.max,
+							baudio->arpeggio.c.notes[chordindex - 1].k);
 					 */
 					if (++chordindex > baudio->arpeggio.c.max)
 						break;
 				}
-				domidiNoteOn(audiomain, msg, baudio, key);
+				doMidiNoteOn(audiomain, msg, baudio, key);
 			}
 		}
 
@@ -857,7 +981,10 @@ allNotesOff(audioMain *audiomain, int channel)
 			|| ((voice->baudio != NULL)
 				&& ((voice->baudio->midichannel == channel)
 					|| (voice->baudio->midichannel == BRISTOL_CHAN_OMNI))))
-			voice->flags |= BRISTOL_KEYOFF;
+		{
+			voice->flags |= BRISTOL_KEYOFF|BRISTOL_KEYDONE;
+			voice->flags &= ~BRISTOL_KEYSUSTAIN;
+		}
 
 		doMNL(voice, 0, 0, BRISTOL_ALL_NOTES_OFF);
 		voice = voice->next;
@@ -867,3 +994,17 @@ allNotesOff(audioMain *audiomain, int channel)
 		printf("midi all notes off\n");
 }
 
+#ifndef BRISTOL_SEMAPHORE
+void
+rbMidiNote(audioMain *audiomain, bristolMidiMsg *msg)
+{
+	if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG2)
+		printf("rbMidiNote(%x) %x\n", msg->command, msg->params.key.flags);
+
+	if ((msg->command & MIDI_COMMAND_MASK) == MIDI_NOTE_ON)
+		rbMidiNoteOn(audiomain, msg);
+	else if ((msg->command & MIDI_COMMAND_MASK) == MIDI_NOTE_OFF)
+		rbMidiNoteOff(audiomain, msg);
+}
+#endif
+
