@@ -375,12 +375,13 @@ midiThread(audioMain *audiomain)
 	{
 		printf("Error opening midi device %s/%i, exiting midi thread\n",
 			device, audiomain->midiHandle);
+		printf("Bristol cannot operate without a midi interface. Terminating\n");
 		audiomain->atReq = BRISTOL_REQSTOP;
 		exitReq = 1;
 		bristolMidiClose(audiomain->controlHandle);
 		pthread_exit(&exitstatus);
 	} else
-		printf("opened device socket\n");
+		printf("opened midi device %s\n", device);
 
 #ifdef DEBUG
 	printf("opened midi device: %i/%i\n",
