@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -216,7 +215,7 @@ static int operate(bristolOP *operator,
 	void *lcl)
 {
 	register bristolVOXlocal *local = lcl;
-	register int obp, count;
+	register int obp = 0, count;
 	register float *ib, *ob, *pb, *pt, *wt, wtp, gain;
 	bristolVOX *specs;
 
@@ -236,9 +235,20 @@ static int operate(bristolOP *operator,
 	wtp = local->wtp;
 
 	if (voice->flags & BRISTOL_KEYON)
+	{
 		wtp = 0;
+		if ((~voice->flags & BRISTOL_KEYREON) &&
+			(voice->offset > 0) && (voice->offset < count))
+		{
+			if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+				printf("envelope trigger offset %i frames\n", voice->offset);
 
-	for (obp = 0; obp < count;)
+			//count += voice->offset;
+			obp += voice->offset;
+		}
+	}
+
+	for (; obp < count;)
 	{
 		/*
 		 * Take a sample from the wavetable into the output buffer. This 
@@ -376,6 +386,7 @@ voxdcoinit(bristolOP **operator, int index, int samplerate, int samplecount)
 	return(*operator);
 }
 
+/*
 static void filltriwave(register float *mem, register int count,
 register double reach)
 {
@@ -388,6 +399,7 @@ register double reach)
 		return;
 	}
 }
+*/
 
 /*
  * Hm, just in case you could not tell, this is a phase distortion oscillator.
