@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -51,8 +50,10 @@ typedef struct BrightonSize {
 #define BRIGHTON_KEYRELEASE 6
 #define BRIGHTON_ENTER 7
 #define BRIGHTON_LEAVE 8
-#define BRIGHTON_CONFIGURE 9
-#define BRIGHTON_CONFIGURE_REQ 10
+#define BRIGHTON_FOCUS_IN 9
+#define BRIGHTON_FOCUS_OUT 10
+#define BRIGHTON_CONFIGURE 22 // 9
+#define BRIGHTON_CONFIGURE_REQ 23 // 10
 #define BRIGHTON_EXPOSE 11
 #define BRIGHTON_GEXPOSE 12
 #define BRIGHTON_CLIENT 15
@@ -66,6 +67,9 @@ typedef struct BrightonSize {
 #define BRIGHTON_BUTTON2 2
 #define BRIGHTON_BUTTON3 3
 
+#define BRIGHTON_MOD_SHIFT 1
+#define BRIGHTON_MOD_CONTROL 4
+
 /*
  * This is an oversimplified conversion of the XEvent strcture for brighton use
  */
@@ -81,6 +85,7 @@ typedef struct BrightonEvent {
 	float value;
 	int intvalue;
 	void *m;
+	//int ox, oy;
 } brightonEvent;
 
 
