@@ -225,16 +225,19 @@
 /*
  * Audio interface types. The loest bytes is resevered.
  *
- * These are for audiomain.flags:
+ * These are for audiomain.flags, there are 8 drivers available of which are
+ * defined, one is deprecated leaving six. The Pulse has two interfaces one of
+ * which may eventually be discarded. They are pulse simple and pulse, the 
+ * latter is a full callback interface, multithreaded and rather heavy on CPU.
  */
 #define BRISTOL_AUDIOMASK		0xff000000
 #define BRISTOL_ALSA			0x01000000
 #define BRISTOL_OSS				0x02000000
 #define BRISTOL_SLAB			0x04000000 /* should deprecte this */
 #define BRISTOL_JACK			0x08000000
-#define BRISTOL_DSSI			0x10000000 /* Unlikely to be supported - LV2? */
-#define BRISTOL_DUMMY			0x20000000
-#define BRISTOL_GSTREAMER		0x40000000
+#define BRISTOL_PULSE			0x10000000 /* Threaded */
+#define BRISTOL_PULSE_S			0x20000000 /* Simple - lots of latency */
+#define BRISTOL_DUMMY			0x40000000
 /* This appears to collide */
 #define BRISTOL_AUDIOWAIT		0x00002000
 
