@@ -43,5 +43,9 @@
 
 #include "slabcdefs.h"
 
+#ifdef BRISTOL_PA
+#include "bristolpulse.h"
+#endif
+
 #endif /* SLAB_ENGINE */
 
