@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,7 +30,6 @@
 #include "brightoninternals.h"
 
 extern int brightonPanelLocation();
-extern int brightonRegisterController();
 
 int
 destroyButton(brightonDevice *dev)
@@ -76,32 +74,32 @@ displaybutton(brightonDevice *dev)
 	if (dev->bwin->app->resources[dev->panel].devlocn[dev->index].flags
 		& (BRIGHTON_THREEWAY|BRIGHTON_FIVEWAY))
 	{
-		if (dev->value == 0)
+		if ((dev->value >= 0) && (dev->value <= 0.5))
 		{
 			brightonStretch(dev->bwin, dev->imagec,
 				dev->bwin->dlayer,
 				dev->x + dev->bwin->app->resources[dev->panel].sx,
 				dev->y + dev->bwin->app->resources[dev->panel].sy,
 				dev->width, dev->height, flags);
-		} else if (dev->value == 1) {
+		} else if ((dev->value > 0.5) && (dev->value <= 1.5)) {
 			brightonStretch(dev->bwin, dev->image2,
 				dev->bwin->dlayer,
 				dev->x + dev->bwin->app->resources[dev->panel].sx,
 				dev->y + dev->bwin->app->resources[dev->panel].sy,
 				dev->width, dev->height, flags);
-		} else if (dev->value == 2) {
+		} else if ((dev->value > 1.5) && (dev->value <= 2.5)) {
 			brightonStretch(dev->bwin, dev->image,
 				dev->bwin->dlayer,
 				dev->x + dev->bwin->app->resources[dev->panel].sx,
 				dev->y + dev->bwin->app->resources[dev->panel].sy,
 				dev->width, dev->height, flags);
-		} else if (dev->value == 3) {
+		} else if ((dev->value > 2.5) && (dev->value <= 3.5)) {
 			brightonStretch(dev->bwin, dev->image3,
 				dev->bwin->dlayer,
 				dev->x + dev->bwin->app->resources[dev->panel].sx,
 				dev->y + dev->bwin->app->resources[dev->panel].sy,
 				dev->width, dev->height, flags);
-		} else if (dev->value == 4) {
+		} else if ((dev->value > 3.5) && (dev->value <= 4.5)) {
 			brightonStretch(dev->bwin, dev->image4,
 				dev->bwin->dlayer,
 				dev->x + dev->bwin->app->resources[dev->panel].sx,
