@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -27,10 +26,16 @@
 #include "brightonX11.h"
 
 static float opacity = 1.0;
-static int shiftflag = 0;
-static int controlflag = 0;
 
-extern void brightonKeyInput(brightonWindow*, int, int);
+/*
+ * Much of this could do with a rework to make it more generic, at the moment
+ * a lot of the key handlers are very bespoke.
+ */
+
+extern void brightonKeyInput(brightonWindow *, int, int);
+extern void brightonControlKeyInput(brightonWindow *, int, int);
+extern void brightonShiftKeyInput(brightonWindow *, int, int, int);
+extern void brightonControlShiftKeyInput(brightonWindow *, int, int, int);
 extern void printColorCacheStats(brightonWindow *);
 
 extern int brightonMenu();
@@ -40,7 +45,10 @@ brightonKeyPress(brightonWindow *bwin, brightonEvent *event)
 {
 	brightonIResource *panel;
 
-	if ((event->key == 'p') && controlflag)
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonKeyPress(%i)\n", event->key);
+
+	if ((event->key == 'p') && (event->flags & BRIGHTON_MOD_CONTROL))
 	{
 		brightonXpmWrite(bwin, "/tmp/brighton.xpm");
 		printColorCacheStats(bwin);
@@ -55,7 +63,7 @@ brightonKeyPress(brightonWindow *bwin, brightonEvent *event)
 	 * rather trivial reasons before being passed through to the GUI. These
 	 * implicit mappings should be dropped.
 	 */
-	if ((event->key == 't') && (controlflag))
+	if ((event->key == 't') && (event->flags & BRIGHTON_MOD_CONTROL))
 	{
 		float hold;
 
@@ -63,17 +71,13 @@ brightonKeyPress(brightonWindow *bwin, brightonEvent *event)
 		bwin->opacity = opacity;
 		opacity = hold;
 		brightonFinalRender(bwin, 0, 0, bwin->width, bwin->height);
-	} else if ((event->key == 65508) || (event->key == 65507)) {
-		controlflag = 1;
-	} else if ((event->key == 65505) || (event->key == 65506)) {
-		shiftflag = 1;
-	} else if ((event->key == 'o') && (controlflag)) {
+	} else if ((event->key == 'o') && (event->flags & BRIGHTON_MOD_CONTROL)) {
 		/*
 		 * 'O' should be to make more opaque, 'o' more transparent. This is not
 		 * as * trivial as it seems since the shift key is always a separate
 		 * event.
 		 */
-		if (shiftflag)
+		if (event->flags & BRIGHTON_MOD_SHIFT)
 		{
 			if (bwin->opacity == 1.0f)
 				bwin->opacity = 0.2f;
@@ -110,7 +114,14 @@ brightonKeyPress(brightonWindow *bwin, brightonEvent *event)
 	 * mapping. We should only really do this from the keyboard panel, but that
 	 * if for later study.
 	 */
-	if (controlflag == 0)
+	if ((event->flags & BRIGHTON_MOD_CONTROL)
+		&& (event->flags & BRIGHTON_MOD_SHIFT))
+		brightonControlShiftKeyInput(bwin, event->key, 1, event->flags);
+	else if (event->flags & BRIGHTON_MOD_CONTROL)
+		brightonControlKeyInput(bwin, event->key, 1);
+	else if (event->flags & BRIGHTON_MOD_SHIFT)
+		brightonShiftKeyInput(bwin, event->key, 1, event->flags);
+	else
 		brightonKeyInput(bwin, event->key, 1);
 
 	return(0);
@@ -121,12 +132,8 @@ brightonKeyRelease(brightonWindow *bwin, brightonEvent *event)
 {
 	brightonIResource *panel;
 
-/*	printf("brightonKeyRelease(%i)\n", event->key); */
-
-	if ((event->key == 65508) || (event->key == 65507))
-		controlflag = 0;
-	else if ((event->key == 65505) || (event->key == 65506))
-		shiftflag = 0;
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonKeyRelease(%i)\n", event->key);
 
 	if ((bwin->flags & BRIGHTON_DEV_ACTIVE) && (bwin->activepanel != 0))
 		bwin->activepanel->configure(bwin, bwin->activepanel, event);
@@ -137,14 +144,18 @@ brightonKeyRelease(brightonWindow *bwin, brightonEvent *event)
 			panel->configure(bwin, panel, event);
 	}
 
-	if (controlflag == 0)
+	if (~event->flags & BRIGHTON_MOD_CONTROL)
 		brightonKeyInput(bwin, event->key, 0);
+
 	return(0);
 }
 
 int
 brightonButtonPress(brightonWindow *bwin, brightonEvent *event)
 {
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonButtonPress(%i)\n", event->key);
+
 	if (event->key == BRIGHTON_BUTTON3) {
 			brightonMenu(bwin, event->x, event->y, 100, 200);
 		return(0);
@@ -173,8 +184,14 @@ brightonButtonPress(brightonWindow *bwin, brightonEvent *event)
 int
 brightonButtonRelease(brightonWindow *bwin, brightonEvent *event)
 {
-/*printf("brightonButtonRelease(%x, %x, %x)\n", bwin, bwin->activepanel, */
-/*bwin->activepanel->configure); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+	{
+		if ((bwin) && (bwin->activepanel))
+			printf("brightonButtonRelease(%p, %p, %p)\n",
+				bwin,
+				bwin->activepanel,
+				bwin->activepanel->configure); 
+	}
 
 	event->command = BRIGHTON_BUTTONRELEASE;
 
@@ -203,18 +220,163 @@ brightonMotionNotify(brightonWindow *bwin, brightonEvent *event)
 	return(0);
 }
 
+static void
+brightonFillRatios(brightonWindow *win)
+{
+	float wfact, hfact;
+
+	wfact = ((float) win->display->width) * 0.95
+		/ ((float) win->template->width);
+	hfact = ((float) win->display->height) * 0.95
+		/ ((float) win->template->height);
+
+	if (hfact > wfact) {
+		win->maxw = win->display->width * 0.95;
+		win->minw = win->template->width;
+		win->minh = win->template->height;
+		win->maxh = win->template->height * wfact;
+	} else {
+		win->maxh = win->display->height * 0.95;
+		win->minw = win->template->width;
+		win->minh = win->template->height;
+		win->maxw = win->template->width * hfact;
+	}
+}
+
 int
 brightonEnterNotify(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonEnterNotify()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonEnterNotify()\n");
+
+#if (BRIGHTON_HAS_ZOOM == 1)
+	if (bwin->flags & BRIGHTON_AUTOZOOM)
+	{
+		if (bwin->flags & BRIGHTON_DEBUG)
+			printf("AutoZoom\n");
+
+		// Make sure we are initted 
+		if ((bwin->minh == 0) || (bwin->maxh == 0))
+			brightonFillRatios(bwin);
+
+		bwin->display->flags |= BRIGHTON_ANTIALIAS_5;
+
+		brightonRequestResize(bwin,
+			bwin->template->width,
+			bwin->template->height);
+
+		if (bwin->flags & BRIGHTON_SET_RAISE)
+			BRaiseWindow(bwin->display, bwin);
+	} else
+#endif
+	if (~bwin->flags & BRIGHTON_NO_ASPECT) {
+		float aspect = ((float) bwin->width) / bwin->height;
+
+	    if ((aspect / bwin->aspect < 0.98) || ((aspect / bwin->aspect) > 1.02))
+		{
+			/* Ratio has changed */
+			if (bwin->flags & _BRIGHTON_SET_HEIGHT) {
+				if ((bwin->height * bwin->aspect)
+					< ((brightonDisplay *) bwin->display)->width)
+					bwin->width = bwin->height * bwin->aspect;
+				else {
+					bwin->width =
+						((brightonDisplay *) bwin->display)->width - 10;
+					bwin->height = bwin->width / bwin->aspect;
+				}
+			} else {
+				if ((bwin->width / bwin->aspect)
+					< ((brightonDisplay *) bwin->display)->height)
+					bwin->height = bwin->width / bwin->aspect;
+				else {
+					bwin->height =
+						((brightonDisplay *) bwin->display)->height - 10;
+					bwin->width = bwin->height * bwin->aspect;
+				}
+			}
+
+			if (bwin->flags & BRIGHTON_DEBUG)
+				printf("changed aspect ratio: %f %i %i\n",
+					aspect, bwin->width, bwin->height);
+
+			BResizeWindow(bwin->display, bwin, bwin->width, bwin->height);
+			brightonWorldChanged(bwin, bwin->width, bwin->height);
+		}
+	}
+
+	bwin->flags &= ~_BRIGHTON_SET_HEIGHT;
+
+	if (bwin->flags & BRIGHTON_EXITING)
+	{
+		BAutoRepeat(bwin->display, 1);
+		return(0);
+	}
+
 	BAutoRepeat(bwin->display, 0);
+
 	return(0);
 }
 
 int
 brightonLeaveNotify(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonLeaveNotify()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonLeaveNotify()\n");
+
+#if (BRIGHTON_HAS_ZOOM == 1)
+	if (bwin->flags & BRIGHTON_AUTOZOOM)
+	{
+		if (bwin->flags & BRIGHTON_DEBUG)
+			printf("AutoZoom\n");
+
+		// Make sure we are initted 
+		if ((bwin->minh == 0) || (bwin->maxh == 0))
+			brightonFillRatios(bwin);
+
+		// Flip the window size 
+		brightonRequestResize(bwin, bwin->minw, bwin->minh);
+
+		if (bwin->flags & BRIGHTON_SET_LOWER)
+			BLowerWindow(bwin->display, bwin);
+	} else
+#endif
+	if (~bwin->flags & BRIGHTON_NO_ASPECT) {
+		float aspect = ((float) bwin->width) / bwin->height;
+
+	    if ((aspect / bwin->aspect < 0.98) || ((aspect / bwin->aspect) > 1.02))
+		{
+			/* Ratio has changed */
+			if (bwin->flags & _BRIGHTON_SET_HEIGHT) {
+				if ((bwin->height * bwin->aspect)
+					< ((brightonDisplay *) bwin->display)->width)
+					bwin->width = bwin->height * bwin->aspect;
+				else {
+					bwin->width =
+						((brightonDisplay *) bwin->display)->width - 10;
+					bwin->height = bwin->width / bwin->aspect;
+				}
+			} else {
+				if ((bwin->width / bwin->aspect)
+					< ((brightonDisplay *) bwin->display)->height)
+					bwin->height = bwin->width / bwin->aspect;
+				else {
+					bwin->height =
+						((brightonDisplay *) bwin->display)->height - 10;
+					bwin->width = bwin->height * bwin->aspect;
+				}
+			}
+
+			if (bwin->flags & BRIGHTON_DEBUG)
+				printf("changed aspect ratio: %f %i %i\n",
+					aspect, bwin->width, bwin->height);
+
+			BResizeWindow(bwin->display, bwin, bwin->width, bwin->height);
+			brightonWorldChanged(bwin, bwin->width, bwin->height);
+		}
+	}
+
+	bwin->flags &= ~_BRIGHTON_SET_HEIGHT;
+
 	BAutoRepeat(bwin->display, 1);
 	return(0);
 }
@@ -222,29 +384,39 @@ brightonLeaveNotify(brightonWindow *bwin, brightonEvent *event)
 int
 brightonFocusIn(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonFocusIn()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonFocusIn()\n");
+
+	brightonEnterNotify(bwin, event);
+
 	return(0);
 }
 
 int
 brightonFocusOut(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonFocusOut()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonFocusOut()\n");
+
+	brightonLeaveNotify(bwin, event);
+
 	return(0);
 }
 
 int
 brightonKeymapNotify(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonKeymapNotify()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonKeymapNotify()\n");
 	return(0);
 }
 
 int
 brightonExpose(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonExpose(%i %i %i %i)\n", */
-/*		event->x, event->y, event->w, event->h); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonExpose(%i %i %i %i)\n",
+			event->x, event->y, event->w, event->h);
 
 	BCopyArea(bwin->display, 
 		event->x, event->y, event->w, event->h, event->x, event->y);
@@ -254,28 +426,32 @@ brightonExpose(brightonWindow *bwin, brightonEvent *event)
 int
 brightonGraphicsExpose(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonGraphicsExpose()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonGraphicsExpose()\n");
 	return(0);
 }
 
 int
 brightonNoExpose(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonNoExpose()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonNoExpose()\n");
 	return(0);
 }
 
 int
 brightonVisibilityNotify(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonVisibilityNotify()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonVisibilityNotify()\n");
 	return(0);
 }
 
 int
 brightonCreateNotify(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonCreateNotify()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonCreateNotify()\n");
 	return(0);
 }
 
@@ -288,43 +464,49 @@ brightonDestroyNotify(brightonWindow *bwin, brightonEvent *event)
 	 * represented by this GUI. Engine will terminate when all algos have
 	 * finished.
 	 */
-/*	printf("brightonDestroyNotify()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonDestroyNotify()\n");
 	return(0);
 }
 
 int
 brightonUnmapNotify(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonUnmapNotify()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonUnmapNotify()\n");
 	return(0);
 }
 
 int
 brightonMapNotify(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonMapNotify()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonMapNotify()\n");
 	return(0);
 }
 
 int
 brightonMapRequest(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonMapRequest()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonMapRequest()\n");
 	return(0);
 }
 
 int
 brightonReparentNotify(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonReparentNotify()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonReparentNotify()\n");
 	return(0);
 }
 
 int
 brightonConfigureNotify(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonConfigureNotify(%i, %i, %i, %i)\n", */
-/*		event->x, event->y, event->w, event->h); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonConfigureNotify(%i, %i, %i, %i)\n",
+			event->x, event->y, event->w, event->h);
 
 	if (bwin->flags & BRIGHTON_SET_SIZE)
 	{
@@ -335,29 +517,55 @@ brightonConfigureNotify(brightonWindow *bwin, brightonEvent *event)
 	if ((bwin->width != event->w) || (bwin->height != event->h))
 		brightonWorldChanged(bwin, event->w, event->h);
 
-	if ((bwin->width != event->w) || (bwin->height != event->h))
-		BResizeWindow(bwin->display, bwin, bwin->width, bwin->height);
+	/* This is wrong. */
+//	if ((bwin->width != event->w) || (bwin->height != event->h))
+//		BResizeWindow(bwin->display, bwin, bwin->width, bwin->height);
+
 	return(0);
 }
 
 int
 brightonConfigureRequest(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonConfigureRequest()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonConfigureRequest()\n");
 	return(0);
 }
 
 int
 brightonGravityNotify(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonGravityNotify()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonGravityNotify()\n");
+	return(0);
+}
+
+int
+brightonRequestResize(brightonWindow *bwin, int w, int h)
+{
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonResizeRequest(%i, %i)\n", w, h);
+
+	if ((bwin->width != w) || (bwin->height != h))
+	{
+		if ((bwin->width < w) && (bwin->flags & BRIGHTON_SET_RAISE))
+			BRaiseWindow(bwin->display, bwin);
+
+		/*
+		 * Our size has changed. We need to re-render the background, and then 
+		 * repaint it.
+		 */
+		BResizeWindow(bwin->display, bwin, w, h);
+		brightonWorldChanged(bwin, w, h);
+	}
 	return(0);
 }
 
 int
 brightonResizeRequest(brightonWindow *bwin, brightonEvent *event)
 {
-/*	printf("brightonResizeRequest()\n"); */
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonResizeRequest(%i, %i)\n", event->w, event->h);
 
 	if ((bwin->width != event->w) || (bwin->height != event->h))
 	{
@@ -378,70 +586,80 @@ brightonResizeRequest(brightonWindow *bwin, brightonEvent *event)
 int
 brightonCirculateNotify(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonCirculateNotify()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonCirculateNotify()\n");
 	return(0);
 }
 
 int
 brightonCirculateRequest(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonCirculateRequest()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonCirculateRequest()\n");
 	return(0);
 }
 
 int
 brightonPropertyNotify(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonPropertyNotify()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonPropertyNotify()\n");
 	return(0);
 }
 
 int
 brightonSelectionClear(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonSelectionClear()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonSelectionClear()\n");
 	return(0);
 }
 
 int
 brightonSelectionRequest(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonSelectionRequest()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonSelectionRequest()\n");
 	return(0);
 }
 
 int
 brightonSelectionNotify(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonSelectionNotify()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonSelectionNotify()\n");
 	return(0);
 }
 
 int
 brightonColormapNotify(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonColormapNotify()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonColormapNotify()\n");
 	return(0);
 }
 
 int
 brightonClientMessage(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonClientRequest()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonClientRequest()\n");
 	return(0);
 }
 
 int
 brightonMappingNotify(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonMappingNotify()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonMappingNotify()\n");
 	return(0);
 }
 
 int
 brightonNullHandler(brightonWindow *bwin, brightonEvent *event)
 {
-	printf("brightonNullHandler()\n");
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonNullHandler()\n");
 	return(0);
 }
 
