@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -74,10 +73,11 @@ brightonInitBitmap(brightonBitmap *bitmap, int value)
 int
 brightonWorldChanged(brightonWindow *bwin, int width, int height)
 {
-	float aspect;
-
-printf("brightonWorldChanged(%i %i %i %i)\n", width, height,
-bwin->width, bwin->height);
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("brightonWorldChanged(%i %i %i %i) %i %i\n",
+			width, height,
+			bwin->width, bwin->height,
+			bwin->template->width, bwin->template->height);
 
 	/*
 	 * We need to maintain aspect ratio. First take the new width, and build
@@ -90,45 +90,28 @@ bwin->width, bwin->height);
 	 * ratio. Required by some window managers to prevent permanant redrawing 
 	 * of the window however it results in some pretty gruesome images if the
 	 * ratio is not a reasonable approximation of the original.
+	 *
+	 * There is some pretty ugly resizing stuff here. Going to propose the app
+	 * will lose aspect ratio as resizing takes place, then on Focus change
+	 * events it will see if aspect should be maintained and do a single resize
+	 * at that point.
 	 */
-	aspect = ((float) width) / height;
+	if (bwin->flags & BRIGHTON_DEBUG)
+		printf("%i %i, %i %i\n", width, height, bwin->width, bwin->height);
 
-	if (bwin->height == height)
+	if (bwin->height != height)
+		bwin->flags |= _BRIGHTON_SET_HEIGHT;
+
+	/* This is the KDE Postage Stamp fix */
+	if ((height < 30) || (width < 50))
 	{
-		/*
-		 * width has changed, calculate new height. If height is our max,
-		 * for this screen, then let it be max, and calculate alternative
-		 * width.
-		 */
-		if ((width / bwin->aspect)
-			< ((brightonDisplay *) bwin->display)->height)
-		{
-			bwin->width = width;
-			bwin->height = bwin->width / bwin->aspect;
-		} else {
-			bwin->height = ((brightonDisplay *) bwin->display)->height - 10;
-			bwin->width = bwin->height * bwin->aspect;
-		}
-	} else {
-		if ((height * bwin->aspect)
-			< ((brightonDisplay *) bwin->display)->width)
-		{
-			bwin->height = height;
-			bwin->width = height * bwin->aspect;
-		} else {
-			bwin->width = ((brightonDisplay *) bwin->display)->width - 10;
-			bwin->height = bwin->width / bwin->aspect;
-		}
+		brightonRequestResize(bwin,
+			bwin->template->width, bwin->template->height);
+		return(0);
 	}
 
-	if (((aspect / bwin->aspect > 0.98) && ((aspect / bwin->aspect) < 1.02))
-		|| (bwin->flags & BRIGHTON_NO_ASPECT))
-	{
-		bwin->width = width;
-		bwin->height = height;
-	} else {
-		bwin->flags |= BRIGHTON_SET_SIZE;
-	}
+	bwin->width = width;
+	bwin->height = height;
 
 	brightonFreeBitmap(bwin, bwin->canvas);
 	brightonFreeBitmap(bwin, bwin->render);
@@ -272,7 +255,7 @@ brightonLogo(brightonWindow *bwin)
 static int
 antialias(brightonWindow *bwin, brightonBitmap *b, int x, int y, float primary)
 {
-	register int nr, ng, nb, dy, color, cpre, cnext, pindex, cpy, cny;
+	register int nr, ng, nb, dy, color = 0, cpre, cnext, pindex, cpy, cny;
 	register brightonPalette *palette = bwin->display->palette;
 	float secondary;
 
@@ -688,17 +671,35 @@ brightonDoFinalRender(brightonWindow *bwin,
 register int x, register int y, register int width, register int height)
 {
 	register int i, j, dy, pindex, aa;
-	register int *src = bwin->canvas->pixels;
-	register int *dest = bwin->render->pixels;
-	register int *device = bwin->dlayer->pixels;
-	register int *shadow = bwin->slayer->pixels;
-	register int *top = bwin->tlayer->pixels;
-	register int *menu = bwin->mlayer->pixels;
+	register int *src;
+	register int *dest;
+	register int *device;
+	register int *shadow;
+	register int *top;
+	register int *menu;
 
 #ifdef STATS2
 	statFunction(0, "Brighton");
 #endif
 
+	if (bwin == NULL)
+		return(0);
+
+	if (bwin->canvas)
+		src = bwin->canvas->pixels;
+	else	
+		return(0);
+
+	if (bwin->render)
+		dest = bwin->render->pixels;
+	else 
+		return(0);
+
+	device = bwin->dlayer->pixels;
+	shadow = bwin->slayer->pixels;
+	top = bwin->tlayer->pixels;
+	menu = bwin->mlayer->pixels;
+
 	if ((width < 0) || (height < 0))
 		return(0);
 
@@ -988,7 +989,7 @@ src->width, src->height, primary);
 	 * We now have the test bitmap, and it should have generated a palette.
 	 * render the bits onto our background, and copy it over.
 	 */
-	for (j = y; j < (y + height); j++)
+	for (j = y; j <= (y + height); j++)
 	{
 		fy = (j - y) * ((float) src->height) / ((float) height);
 		iy = fy;
@@ -1001,7 +1002,7 @@ src->width, src->height, primary);
 		else
 			py = 1.0 - fy;
 
-		for (i = x; i < (x + width); i++)
+		for (i = x; i <= (x + width - 1); i++)
 		{
 			/*
 			 * Take the x relative index into the source bitmap. These are
@@ -1054,7 +1055,7 @@ src->width, src->height, primary);
 				e = c;
 			else {
 				if (isblue(ix + 1 + iy * src->width, palette, pixels))
-					e = scratch[(int) (i + 1 + j * dest->width)];
+					e = scratch[(int) (i + j * dest->width)];
 				else
 					e = pixels[ix + 1 + iy * src->width];
 				if ((e < 0) || (e >= bwin->cmap_size)) continue;
