@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -32,7 +31,6 @@
 #include "brightoninternals.h"
 /*#include "brightonX11.h" */
 
-extern int brightonRegisterController();
 extern void brightonPanelLocation(brightonWindow *, int, int, int, int, int, int);
 
 int
@@ -391,6 +389,11 @@ configure(brightonDevice *dev, brightonEvent *event)
 				break;
 			case 0x6b:
 			case 0xff52:
+				/*
+				 * UP arrow - we need to put some bristol keysyms in here. For
+				 * portability we should not include the X11 headers but map
+				 * our own translations.
+				 */
 				if (dev->flags & BRIGHTON_SHIFTKEY)
 				{
 					if ((dev->value += ((float) 256) / 16384) > 1.0)
@@ -403,6 +406,7 @@ configure(brightonDevice *dev, brightonEvent *event)
 			case 0x6a:
 			case 0xff54:
 				/*
+				 * Down arrow
 				 * We should have this consider stepped rotary motion where a
 				 * single up or down would give a whole step rather than a
 				 * micromovement
@@ -416,6 +420,28 @@ configure(brightonDevice *dev, brightonEvent *event)
 						dev->value = 0;
 				}
 				break;
+			case 0xff51:
+				/* Left arrow */
+				if (dev->flags & BRIGHTON_SHIFTKEY)
+				{
+					if ((dev->value -= ((float) 2048) / 16384) < 0)
+						dev->value = 0;
+				} else {
+					if ((dev->value -= ((float) 32) / 16384) < 0)
+						dev->value = 0;
+				}
+				break;
+			case 0xff53:
+				/* Right arrow */
+				if (dev->flags & BRIGHTON_SHIFTKEY)
+				{
+					if ((dev->value += ((float) 2048) / 16384) > 1.0)
+						dev->value = 1.0;
+				} else {
+					if ((dev->value += ((float) 32) / 16384) > 1.0)
+						dev->value = 1.0;
+				}
+				break;
 		}
 
 		considercallback(dev);
@@ -450,6 +476,34 @@ configure(brightonDevice *dev, brightonEvent *event)
 
 /*printf("Controlled motion from (%i, %i) to (%i, %i): %f\n", */
 /*cx, cy, event->x, event->y, deltax); */
+		} else if (dev->bwin->flags & BRIGHTON_ROTARY_UD) {
+			if (dev->bwin->app->resources[dev->panel].devlocn[dev->index].flags
+				& BRIGHTON_NOTCH)
+			{
+				dev->value = 1.0f - ((float) (event->y)) * 1.1f
+					/ (dev->bwin->height);
+
+				/* If in range 0.45/0.55 - interpret 0.5 */
+				if (dev->value > 0.55)
+					dev->value -= 0.05;
+				else if (dev->value < 0.45)
+					dev->value += 0.05;
+				else
+					dev->value = 0.5;
+
+				if (dev->value < 0.0f)
+					dev->value = 0.0f;
+				else if (dev->value > 1.0f)
+					dev->value = 1.0f;
+			} else {
+				if ((dev->value = 1.0f - ((float) event->y)
+					/ (dev->bwin->height)) < 0)
+					dev->value = 0.0f;
+				else if (dev->value > 1.0f)
+					dev->value = 1.0f;
+			}
+			//printf("%f %i %i, %i %i\n", dev->value,
+				//event->x, event->y, event->ox, event->oy);
 		} else {
 			double angle, diffx, diffy;
 
