@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,27 +30,29 @@
  * This should go into a library, is used from various places.
  */
 void
-bufmerge(register float *buf1, register float gain1, register float *buf2,
+bufmerge(register float *src, register float gain1, register float *dst,
 	register float gain2, register int size)
 {
+	float *buf3 = dst;
+
 	for (;size > 0; size-=16)
 	{
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
-		*buf2++ = *buf2 * gain2 + *buf1++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
+		*dst++ = *buf3++ * gain2 + *src++ * gain1;
 	}
 	/*
 	 * Correctly speaking we should check here to make sure that size is zero
