@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -34,6 +33,7 @@
 /*#define DEBUG */
 
 #include <unistd.h>
+#include <strings.h>
 
 #include "bristolmessages.h"
 #include "bristol.h"
@@ -48,6 +48,10 @@ extern int bristolMidiTCPOpen();
 extern int bristolMidiOSSOpen();
 extern int bristolMidiALSAOpen();
 extern int bristolMidiSeqOpen();
+extern int bristolMidiSeqKeyEvent();
+extern int bristolMidiSeqPressureEvent();
+extern int bristolMidiSeqPPressureEvent();
+extern int bristolMidiSeqCCEvent();
 #ifdef _BRISTOL_JACK_MIDI
 extern int bristolMidiJackOpen();
 #endif
@@ -64,12 +68,11 @@ extern int bristolMidiSanity();
 extern int bristolMidiDevSanity();
 extern int bristolMidiALSARead();
 extern int bristolMidiSeqRead();
-extern int bristolMidiWrite();
 extern int bristolPhysWrite();
 
 extern int initMidiLib();
 
-static char *devname = "localhost";
+static char devname[64] = "localhost";
 
 /*
  * This is the generic code. Calls the desired (configured) drivers.
@@ -80,12 +83,16 @@ void *param)
 {
 	int handle, devnum;
 
-#ifdef DEBUG
-	printf("bristolMidiOpen(%s, %x)\n", dev, flags);
-#endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiOpen(%s, %x)\n", dev, flags);
 
 	if (dev == NULL)
 		dev = devname;
+	else {
+		if ((strncmp(dev, "unix", 4) == 0) && (strlen(dev) > 5)
+			&& (dev[4] == ':'))
+		snprintf(devname, 64, "%s", &dev[5]);
+	}
 
 	initMidiLib(flags);
 
@@ -110,6 +117,7 @@ void *param)
 	if ((handle = bristolMidiFindFreeHandle()) < 0)
 		return(handle);
 
+	bmidi.msgforwarder = NULL;
 	bmidi.handle[handle].handle = handle; /* That looks kind of wierd! */
 	bmidi.handle[handle].state = BRISTOL_MIDI_OK;
 	bmidi.handle[handle].channel = chan;
@@ -134,7 +142,8 @@ void *param)
 		bmidi.handle[handle].param = param;
 		bmidi.handle[handle].flags = bmidi.dev[devnum].flags;
 
-		printf("reusing connection %x\n", bmidi.dev[devnum].flags);
+		if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+			printf("reusing connection %x\n", bmidi.dev[devnum].flags);
 
 		return(handle);
 	}
@@ -192,6 +201,7 @@ void *param)
 	}
 
 	sprintf(bmidi.dev[devnum].name, "%s", dev);
+
 	bmidi.dev[devnum].state = BRISTOL_MIDI_OK;
 	bmidi.dev[devnum].handleCount = 1;
 	bmidi.dev[devnum].bufindex = 0;
@@ -205,31 +215,96 @@ void *param)
 	return(handle);
 }
 
+void
+bristolMidiRegisterForwarder(int (*msgforwarder)())
+{
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("register message forwarder\n");
+
+	bmidi.msgforwarder = msgforwarder;
+}
+
 int
 bristolMidiOption(int handle, int option, int value)
 {
-	if (bristolMidiSanity(handle) < 0)
-		return(bristolMidiSanity(handle));
-
 	switch (option) {
 		case BRISTOL_NRP_DEBUG:
 		{
 			int i;
 
-			if (value == 1)
+			if (bristolMidiSanity(handle) < 0)
+				return(bristolMidiSanity(handle));
+
+			if (value == 0)
 			{
-				for (i = 0; i < BRISTOL_MIDI_DEVCOUNT; i++)
-					if (bmidi.handle[i].dev >= 0)
-						bmidi.dev[bmidi.handle[i].dev].flags
-							|= _BRISTOL_MIDI_DEBUG;
-			} else {
+				bmidi.flags &= ~BRISTOL_BMIDI_DEBUG;
 				for (i = 0; i < BRISTOL_MIDI_DEVCOUNT; i++)
 					if (bmidi.handle[i].dev >= 0)
 						bmidi.dev[bmidi.handle[i].dev].flags
 							&= ~_BRISTOL_MIDI_DEBUG;
+			} else if (value == 1) {
+				for (i = 0; i < BRISTOL_MIDI_DEVCOUNT; i++)
+					if (bmidi.handle[i].dev >= 0)
+						bmidi.dev[bmidi.handle[i].dev].flags
+							|= _BRISTOL_MIDI_DEBUG;
 			}
+			if (value > 5)
+				bmidi.flags |= BRISTOL_BMIDI_DEBUG;
 			break;
 		}
+		case BRISTOL_NRP_FORWARD:
+			/* Disable event forwarding globally */
+			if (value == 0)
+				bmidi.flags &= ~(BRISTOL_MIDI_FORWARD|BRISTOL_MIDI_FHOLD);
+			else
+				bmidi.flags |= (BRISTOL_MIDI_FORWARD|BRISTOL_MIDI_FHOLD);
+			break;
+		case BRISTOL_NRP_MIDI_GO:
+			if (value == 0)
+				bmidi.flags &= ~(BRISTOL_MIDI_GO|BRISTOL_MIDI_FORWARD);
+			else
+				bmidi.flags |= (bmidi.flags & BRISTOL_MIDI_FHOLD)?
+					(BRISTOL_MIDI_GO|BRISTOL_MIDI_FORWARD):BRISTOL_MIDI_GO;
+			break;
+		case BRISTOL_NRP_SYSID_H:
+			bmidi.SysID = (bmidi.SysID & 0x0000ffff)
+				| ((value << 16) & 0xffff0000);
+			break;
+		case BRISTOL_NRP_SYSID_L:
+			bmidi.SysID = (bmidi.SysID & 0xffff0000) | (value & 0x0000ffff);
+			break;
+		case BRISTOL_NRP_REQ_SYSEX:
+			/*
+			 * This is for the local side only and will prevent event forwarding
+			 * to the GUI (primarily). It prevents events being duplicated on
+			 * the receiving end
+			 */
+			if (bristolMidiSanity(handle) < 0)
+				return(bristolMidiSanity(handle));
+
+			if (value == 1)
+				bmidi.handle[handle].flags |= BRISTOL_CONN_SYSEX;
+			else
+				bmidi.handle[handle].flags &= ~BRISTOL_CONN_SYSEX;
+			bmidi.handle[handle].flags |= BRISTOL_CONN_SYSEX;
+			break;
+		case BRISTOL_NRP_REQ_FORWARD:
+			if (bristolMidiSanity(handle) < 0)
+				return(bristolMidiSanity(handle));
+
+			if (value != 0)
+				bmidi.dev[bmidi.handle[handle].dev].flags
+					|= BRISTOL_CONN_FORWARD;
+			else
+				bmidi.dev[bmidi.handle[handle].dev].flags
+					&= ~BRISTOL_CONN_FORWARD;
+			if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+				printf("	Requested forward on handle %i %i %x %i\n",
+					handle,
+					value,
+					bmidi.dev[bmidi.handle[handle].dev].flags,
+					bmidi.handle[handle].dev);
+			break;
 	}
 
 	return(0);
@@ -238,19 +313,27 @@ bristolMidiOption(int handle, int option, int value)
 int
 bristolMidiClose(int handle)
 {
-#ifdef DEBUG
-	printf("bristolMidiClose()\n");
-#endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiClose(%i) %x\n", handle,
+			bmidi.dev[bmidi.handle[handle].dev].flags & BRISTOL_CONNMASK);
 
 	/*
 	 * We close a handle, not a device. We need to clean out the handle, and if
 	 * no other handles have this device open then close the physical device as
 	 * well.
 	 */
-#warning - close handle, and then device if no longer in use.
 	if (bristolMidiSanity(handle) < 0)
 		return(bristolMidiSanity(handle));
 
+	if (bmidi.dev[0].flags & BRISTOL_ACCEPT_SOCKET)
+	{
+		char filename[128];
+
+		snprintf(filename, 128, "/tmp/br.%s", devname);
+
+		unlink(filename);
+	}
+
 	switch (bmidi.dev[bmidi.handle[handle].dev].flags & BRISTOL_CONNMASK) {
 		case BRISTOL_CONN_OSSMIDI:
 			return(bristolMidiOSSClose(handle));
@@ -263,48 +346,20 @@ bristolMidiClose(int handle)
 			return(bristolMidiJackClose(handle));
 #endif
 			break;
+		case BRISTOL_CONN_TCP:
+			return(bristolMidiTCPClose(handle));
+		default:
+			printf("DID NOT CLOSE A HANDLE!!!\n");
 	}
 
 	return(BRISTOL_MIDI_DRIVER);
 }
 
-/*
- * We need to have the ability to have a duplicated handle, take a handle
- * that does not have a callback and give it one. This is a GUI operation
- * only, it allows us to have a non-blocking request to the control handle
- * for event redistribution.
- */
-int
-bristolMidiDupFD(int handle, int flags, int (*callback)(), void *param)
-{
-	int nh, nd;
-
-	if ((nh = bristolMidiFindFreeHandle()) < 0)
-		return(nh);
-	if ((nd = bristolMidiFindDev(NULL)) < 0)
-		return(nd);
-
-	bcopy(&bmidi.handle[handle], &bmidi.handle[nh],
-		sizeof(bristolMidiHandle));
-	bcopy(&bmidi.dev[bmidi.handle[handle].dev], &bmidi.dev[nd],
-		sizeof(bristolMidiDev));
-
-	bmidi.handle[nh].messagemask = flags;
-	bmidi.handle[nh].callback = callback;
-	bmidi.handle[nh].param = param;
-	bmidi.handle[nh].dev = nd;
-
-	bmidi.dev[nd].flags |= BRISTOL_CONN_NBLOCK;
-
-	return(nh);
-}
-
 int
 bristolMidiDevRead(int dev, bristolMidiMsg *msg)
 {
-#ifdef DEBUG
-	printf("bristolMidiDevRead(%i)\n", dev);
-#endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiDevRead(%i)\n", dev);
 
 	if (bristolMidiDevSanity(dev) < 0)
 		return(bristolMidiDevSanity(dev));
@@ -331,29 +386,45 @@ bristolGetMidiFD(int handle)
 	return(bmidi.handle[handle].dev);
 }
 
+static bristolMidiMsg post;
+
+void
+bristolMidiPost(bristolMidiMsg *msg)
+{
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("post %i bytes\n", (int) sizeof(bristolMidiMsg));
+	bcopy(msg, &post, sizeof(bristolMidiMsg));
+}
+
 int
 bristolMidiRead(int handle, bristolMidiMsg *msg)
 {
+	int c = 50;
+
 #ifdef DEBUG
-	printf("bristolMidiRead(%i): %i/%i\n", handle,
-		bmidi.handle[handle].dev, bmidi.dev[bmidi.handle[handle].dev].fd);
 #endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiRead(%i): %i/%i\n", handle,
+			bmidi.handle[handle].dev, bmidi.dev[bmidi.handle[handle].dev].fd);
 
 	if (bristolMidiSanity(handle) < 0)
 		return(bristolMidiSanity(handle));
 
-	msg->command = 0xff;
+//	msg->command = 0xff;
 
 	if (bmidi.handle[handle].callback == NULL)
 	{
 		while (msg->command == 0xff)
 		{
-/*printf("reading type %x\n", bmidi.dev[bmidi.handle[handle].dev].flags); */
+			if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+				printf("reading type %x\n",
+					bmidi.dev[bmidi.handle[handle].dev].flags); 
+
 			switch (bmidi.dev[bmidi.handle[handle].dev].flags
 				& BRISTOL_CONNMASK)
 			{
-				case BRISTOL_CONN_MIDI:
 				case BRISTOL_CONN_TCP:
+				case BRISTOL_CONN_MIDI:
 				case BRISTOL_CONN_OSSMIDI:
 					if (bristolMidiALSARead(bmidi.handle[handle].dev, msg) < 0)
 						return(-1);
@@ -368,8 +439,41 @@ bristolMidiRead(int handle, bristolMidiMsg *msg)
 		return(BRISTOL_MIDI_OK);
 	} else {
 		switch (bmidi.dev[handle].flags & BRISTOL_CONNMASK) {
-			case BRISTOL_CONN_MIDI:
 			case BRISTOL_CONN_TCP:
+				/*
+				 * If this is the TCP port we should have a wait loop for a
+				 * response from the engine, the event comes in another thread.
+				 */
+				if (bmidi.dev[handle].fd < 0)
+					return(BRISTOL_MIDI_DRIVER);
+
+				while (post.channel == 0xff)
+				{
+					usleep(100000);
+
+					if (--c <= 0) {
+						int i;
+
+						if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+							printf("MIDI/TCP read failure\n");
+
+						for (i = 0; i < BRISTOL_MIDI_DEVCOUNT; i++)
+						{
+							if ((bmidi.dev[i].fd > 0)
+								&& ((bmidi.dev[i].flags & BRISTOL_CONTROL_SOCKET) == 0)
+								&& (bmidi.dev[i].flags & BRISTOL_CONN_TCP))
+								bristolMidiTCPClose(bmidi.dev[i].fd);
+						}
+
+						printf("closing down TCP sockets\n");
+
+						return(BRISTOL_MIDI_DRIVER);
+					}
+				}
+				bcopy(&post, msg, sizeof(bristolMidiMsg));
+				post.channel = 0xff;
+				return(BRISTOL_MIDI_OK);
+			case BRISTOL_CONN_MIDI:
 			case BRISTOL_CONN_OSSMIDI:
 				return(bristolMidiALSARead(bmidi.handle[handle].dev, msg));
 			case BRISTOL_CONN_SEQ:
@@ -383,7 +487,7 @@ bristolMidiRead(int handle, bristolMidiMsg *msg)
 int
 bristolMidiTerminate()
 {
-	printf("Terminate MIDI signalling\n");
+	printf("terminate MIDI signalling\n");
 	/*
 	 * We need this since the library never actually returns when we are
 	 * using the midiCheck routines. We are going to close all handles.
@@ -392,12 +496,19 @@ bristolMidiTerminate()
 
 	if (bmidi.dev[0].flags & BRISTOL_CONTROL_SOCKET)
 	{
+		char filename[128];
+
 		/*
 		 * We should send an all_notes_off and terminate the socket layer with
 		 * a REQSTOP
 		 */
+		sleep(1);
 		close(bmidi.dev[0].fd);
-		unlink(BRISTOL_SOCKNAME);
+		bmidi.dev[0].fd = -1;
+
+		snprintf(filename, 128, "/tmp/bristol.%s", devname);
+
+		unlink(filename);
 	}
 
 	/*
@@ -413,21 +524,44 @@ bristolMidiRawWrite(int dev, bristolMidiMsg *msg, int size)
 	if (bristolMidiDevSanity(dev) < 0)
 		return(bristolMidiDevSanity(dev));
 
-#ifdef DEBUG
-	printf("bristolMidiRawWrite %i/%i, %i\n", dev, bmidi.dev[dev].fd, size);
-#endif
-printf("bristolMidiRawWrite (%x) %x/%x/%x\n",
-msg->command,
-msg->channel,
-msg->params.key.key,
-msg->params.key.velocity);
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiRawWrite (%x) %x/%i\n",
+			msg->command,
+			msg->channel,
+			size);
 
 	if (bristolPhysWrite(bmidi.dev[dev].fd,
 		(unsigned char *) &msg->command, 1) != 0)
 		return(1);
-	if (bristolPhysWrite(bmidi.dev[dev].fd,
-		(unsigned char *) &msg->params, size - 1) != 0)
-		return(1);
+
+	if (msg->command != MIDI_SYSEX)
+	{
+		if (bristolPhysWrite(bmidi.dev[dev].fd,
+			(unsigned char *) &msg->params, size - 1) != 0)
+			return(1);
+	} else {
+		if (msg->params.bristol.msgType < 8)
+		{
+			if (bristolPhysWrite(bmidi.dev[dev].fd,
+				(unsigned char *) &msg->params, size) != 0)
+				return(1);
+		} else {
+			if (bristolPhysWrite(bmidi.dev[dev].fd,
+				(unsigned char *) &msg->params, 12) != 0)
+				return(1);
+			if (bristolPhysWrite(bmidi.dev[dev].fd,
+				(unsigned char *) msg->params.bristolt2.data,
+					msg->params.bristol.msgLen - 12) != 0)
+				return(1);
+		}
+	}
+
+	if (msg->command == MIDI_SYSEX)
+	{
+		unsigned char comm = MIDI_EOS;
+
+		return(bristolPhysWrite(bmidi.dev[dev].fd, (unsigned char *) &comm, 1));
+	}
 
 	return(0);
 }
@@ -440,9 +574,8 @@ bristolMidiWrite(int dev, bristolMsg *msg, int size)
 	if (bristolMidiDevSanity(dev) < 0)
 		return(bristolMidiDevSanity(dev));
 
-#ifdef DEBUG
-	printf("bristolMidiWrite %i/%i, %i\n", dev, bmidi.dev[dev].fd, size);
-#endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiWrite %i/%i, %i\n", dev, bmidi.dev[dev].fd, size);
 
 	byte = MIDI_SYSEX;
 	if (bristolPhysWrite(bmidi.dev[dev].fd, &byte, 1) != 0)
@@ -470,9 +603,16 @@ bristolMidiControl(int handle, int channel, int operator, int controller,
 	ctr = controller;
 	val = value & 0x7f;
 
-	bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &comm, 1);
-	bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &ctr, 1);
-	bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &val, 1);
+	switch (bmidi.dev[bmidi.handle[handle].dev].flags & BRISTOL_CONNMASK) {
+		case BRISTOL_CONN_SEQ:
+			return(bristolMidiSeqCCEvent(bmidi.handle[handle].dev,
+				operator, channel, controller, value));
+		default:
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &comm, 1);
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &ctr, 1);
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &val, 1);
+			break;
+	}
 
 	return(0);
 }
@@ -510,10 +650,9 @@ bristolKeyEvent(int handle, int op, int channel, int key, int velocity)
 	key &= 0x7f;
 	velocity &= 0x7f;
 
-#ifdef DEBUG
-	printf("key event ch: %i, key: %i over fd %i\n", channel, key,
-		bmidi.dev[bmidi.handle[handle].dev].fd);
-#endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("key event ch: %i, key: %i over fd %i\n", channel, key,
+			bmidi.dev[bmidi.handle[handle].dev].fd);
 
 	if (bristolMidiSanity(handle) < 0)
 		return(bristolMidiSanity(handle));
@@ -523,9 +662,72 @@ bristolKeyEvent(int handle, int op, int channel, int key, int velocity)
 	else
 		comm = MIDI_NOTE_OFF | channel;
 
-	bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &comm, 1);
-	bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &key, 1);
-	bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &velocity, 1);
+	switch (bmidi.dev[bmidi.handle[handle].dev].flags & BRISTOL_CONNMASK) {
+		case BRISTOL_CONN_SEQ:
+			return(bristolMidiSeqKeyEvent(bmidi.handle[handle].dev,
+				op, channel, key, velocity));
+		default:
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &comm, 1);
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &key, 1);
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &velocity, 1);
+			break;
+	}
+
+	return(0);
+}
+
+int
+bristolPolyPressureEvent(int handle, int op, int channel, int key, int press)
+{
+	unsigned char comm;
+
+	comm = 0xa0 | (channel & 0x0f);
+	key &= 0x7f;
+	press &= 0x7f;
+
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("pressure ch: %i, pressure: %i over fd %i\n", channel, press,
+			bmidi.dev[bmidi.handle[handle].dev].fd);
+
+	if (bristolMidiSanity(handle) < 0)
+		return(bristolMidiSanity(handle));
+
+	switch (bmidi.dev[bmidi.handle[handle].dev].flags & BRISTOL_CONNMASK) {
+		case BRISTOL_CONN_SEQ:
+			return(bristolMidiSeqPPressureEvent(bmidi.handle[handle].dev,
+				op, channel, key, press));
+		default:
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &comm, 1);
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &key, 1);
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &press, 1);
+	}
+
+	return(0);
+}
+
+int
+bristolPressureEvent(int handle, int op, int channel, int press)
+{
+	unsigned char comm;
+
+	comm = 0xd0 | (channel & 0x0f);
+	press &= 0x7f;
+
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("pressure ch: %i, pressure: %i over fd %i\n", channel, press,
+			bmidi.dev[bmidi.handle[handle].dev].fd);
+
+	if (bristolMidiSanity(handle) < 0)
+		return(bristolMidiSanity(handle));
+
+	switch (bmidi.dev[bmidi.handle[handle].dev].flags & BRISTOL_CONNMASK) {
+		case BRISTOL_CONN_SEQ:
+			return(bristolMidiSeqPressureEvent(bmidi.handle[handle].dev,
+				op, channel, press));
+		default:
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &comm, 1);
+			bristolPhysWrite(bmidi.dev[bmidi.handle[handle].dev].fd, &press, 1);
+	}
 
 	return(0);
 }
@@ -604,10 +806,15 @@ bristolMidiSendMsg(int handle, int channel, int operator, int controller,
 {
 	bristolMsg msg;
 
-#ifdef DEBUG
-	printf("bristolMidiSendMsg(%i, %i, %i, %i, %i)\n",
-		handle, channel, operator, controller, value);
-#endif
+	bzero(&msg, sizeof(bristolMsg));
+
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+	{
+		printf("bristolMidiSendMsg(%i, %i, %i, %i, %i)\n",
+			handle, channel, operator, controller, value);
+		printf("%i %i %i\n", handle, bmidi.handle[handle].dev,
+			bmidi.dev[bmidi.handle[handle].dev].fd);
+	}
 
 	if ((value > C_RANGE_MIN_1) || (value < 0))
 		printf("controller %i/%i value %i outside range\n",
@@ -626,19 +833,18 @@ bristolMidiSendMsg(int handle, int channel, int operator, int controller,
 	}
 	if ((operator == BRISTOL_EVENT_KEYON) || (operator == BRISTOL_EVENT_KEYOFF))
 	{
-		bristolKeyEvent(bmidi.handle[handle].dev, operator,
-			channel, value & BRISTOL_PARAMMASK, 120);
-		return(0);
+		return(bristolKeyEvent(bmidi.handle[handle].dev, operator,
+			channel, value & BRISTOL_PARAMMASK, 120));
 	}
 	/*
 	 * We are going to send this through to the control port as a basic format
 	 * message. The parameter is not going to be scaled, just given as a value
 	 * from 0 to 16383. The receiver can decide how to scale the value.
 	 */
-	msg.SysID = 83;
-	msg.L = 76;
-	msg.a = 97;
-	msg.b = 98;
+	msg.SysID = (bmidi.SysID >> 24) & 0x0ff;
+	msg.L = (bmidi.SysID >> 16) & 0x0ff;
+	msg.a = (bmidi.SysID >> 8) & 0x0ff;
+	msg.b = bmidi.SysID & 0x0ff;
 
 	msg.msgLen = sizeof(bristolMsg);
 	msg.msgType = MSG_TYPE_PARAM;
@@ -650,6 +856,10 @@ bristolMidiSendMsg(int handle, int channel, int operator, int controller,
 	msg.valueLSB = value & 0x007f;
 	msg.valueMSB = (value >> 7) & 0x007f;
 
+//	bristolMsgPrint(&msg);
+
+	post.channel = 0xff;
+
 	bristolMidiWrite(bmidi.handle[handle].dev, &msg, sizeof(bristolMsg));
 
 	return(0);
