@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -50,10 +49,10 @@ int (*callback)(), void *param, int dev, int handle)
 #if (BRISTOL_HAS_ALSA == 1)
 #if (SND_LIB_MAJOR == 1)
 	int nfds;
+	snd_seq_port_info_t *port_info;
 
-#ifdef DEBUG
-	printf("bristolMidiALSAOpen(%s)\n", devname);
-#endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiALSAOpen(%s)\n", devname);
 
 	bmidi.dev[dev].flags = SND_SEQ_OPEN_INPUT;
 	if (snd_rawmidi_open(&bmidi.dev[dev].driver.alsa.handle, NULL,
@@ -66,6 +65,19 @@ int (*callback)(), void *param, int dev, int handle)
 		return(BRISTOL_MIDI_DRIVER);
 	}
 
+	snd_seq_port_info_alloca (&port_info);
+	snd_seq_port_info_set_name (port_info, "bristol input port");
+	snd_seq_port_info_set_capability (port_info,
+		SND_SEQ_PORT_CAP_WRITE|SND_SEQ_PORT_CAP_SUBS_WRITE);
+	snd_seq_port_info_set_type (port_info,
+		SND_SEQ_PORT_TYPE_MIDI_GENERIC|SND_SEQ_PORT_TYPE_SYNTHESIZER);
+		/*|SND_SEQ_PORT_TYPE_SPECIFIC); */
+
+	if (snd_seq_create_port (bmidi.dev[dev].driver.seq.handle, port_info))
+	      printf ("error creating alsa port\n");
+	else if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+	      printf ("created alsa port\n");
+
 	/*
 	 * We need to get a file descriptor for this interface, since we are going
 	 * to use it with select() over several interfaces.
@@ -98,9 +110,8 @@ int (*callback)(), void *param, int dev, int handle)
 int
 bristolMidiALSAClose(int handle)
 {
-#ifdef DEBUG
-	printf("bristolMidiALSAClose()\n");
-#endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiALSAClose()\n");
 
 #if (BRISTOL_HAS_ALSA == 1)
 #if (SND_LIB_MAJOR == 1)
@@ -138,19 +149,71 @@ checkcallbacks(bristolMidiMsg *msg)
 {
 	int i, message = 1 << ((msg->command & 0x70) >> 4);
 
-//printf("msg from %i, sid %i\n", msg->params.bristol.from,
-//msg->params.bristol.channel);
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("msg from %i, chan %i, %i bytes\n", msg->params.bristol.from,
+			msg->params.bristol.channel, msg->params.bristol.msgLen);
 
 	for (i = 0; i < BRISTOL_MIDI_HANDLES; i++)
 	{
-		if (bmidi.dev[bmidi.handle[i].dev].flags & BRISTOL_ACCEPT_SOCKET)
+		if ((bmidi.dev[bmidi.handle[i].dev].flags & BRISTOL_ACCEPT_SOCKET)
+			|| (bmidi.dev[i].flags & BRISTOL_CONN_JACK)
+			|| (bmidi.handle[i].state < 0))
 			continue;
 
-		if (bmidi.handle[i].state < 0)
-			continue;
+		if ((bmidi.dev[bmidi.handle[i].dev].fd > 0)
+			&& (bmidi.flags & BRISTOL_MIDI_FORWARD)
+			&& (bmidi.flags & BRISTOL_MIDI_GO)
+			//&& (~bmidi.dev[i].flags & BRISTOL_CONN_JACK)
+			&& (~bmidi.dev[msg->params.bristol.from].flags & BRISTOL_CONN_TCP)
+			&& (bmidi.dev[bmidi.handle[i].dev].flags & BRISTOL_CONN_TCP)
+			&& (bmidi.dev[bmidi.handle[i].dev].flags & BRISTOL_CONN_FORWARD)
+			&& (bmidi.handle[i].dev >= 0)
+			&& (msg->params.bristol.msgLen != 0))
+		{
+			if (bmidi.dev[bmidi.handle[i].dev].flags & _BRISTOL_MIDI_DEBUG)
+				printf(
+					"candidate for forwarding: %i: %i -> %i (%x %x: %i %i)\n",
+					i,
+					msg->params.bristol.from,
+					bmidi.handle[i].dev,
+					bmidi.dev[msg->params.bristol.from].flags,
+					bmidi.dev[bmidi.handle[i].dev].flags,
+					bmidi.dev[bmidi.handle[i].dev].fd,
+					msg->params.bristol.msgLen);
+			if (bmidi.msgforwarder != NULL) {
+				msg->mychannel = bmidi.handle[i].dev;
+				bmidi.msgforwarder(msg);
+			} else if (bristolMidiRawWrite(bmidi.handle[i].dev, msg,
+				msg->params.bristol.msgLen) != 0)
+				printf("forward failed\n");
+		}
+		/*
+			printf("Not a forwarding candidate\n%i %x: %x %x %x %x %x %i\n",
+				i, //7
+				bmidi.flags,
+				bmidi.flags & BRISTOL_MIDI_FORWARD, //20000000
+				bmidi.flags & BRISTOL_MIDI_GO, //80000000
+				~bmidi.dev[msg->params.bristol.from].flags & BRISTOL_CONN_TCP,
+				bmidi.dev[bmidi.handle[i].dev].flags & BRISTOL_CONN_TCP,
+				bmidi.dev[bmidi.handle[i].dev].flags & BRISTOL_CONN_FORWARD,
+				msg->params.bristol.msgLen);
+		*/
 
 		if (bmidi.handle[i].callback == NULL)
+		{
+			if (bmidi.dev[bmidi.handle[i].dev].flags & _BRISTOL_MIDI_DEBUG)
+				printf("null callback\n");
+			/*
+			 * We should not really have null callbacks. The only use for it
+			 * was for blocking read operations for the GUI to get acks back
+			 * from the engine.
+			 *
+			 * Registrations with null callback should be given a default
+			 * handler which flags the message as 'new' for the reader thread
+			 * to take it and return.
+			 */
 			continue;
+		}
 
 		if (bmidi.handle[i].messagemask & message)
 		{
@@ -162,12 +225,21 @@ checkcallbacks(bristolMidiMsg *msg)
 				 */
 				if (msg->params.bristol.from == bmidi.handle[i].dev)
 				{
+					msg->params.bristol.from = i;
 					bmidi.handle[i].callback(msg, bmidi.handle[i].param);
 					return;
 				}
 			} else {
 				unsigned char hold = msg->params.bristol.from;
 
+				if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+					printf("callback non sysex: %i %x\n",
+						i, bmidi.handle[i].flags);
+
+				if (((~bmidi.flags & BRISTOL_MIDI_GO)
+					&& (((msg->command & ~MIDI_STATUS_MASK) >> 4) < 2))
+					|| (bmidi.handle[i].flags & BRISTOL_CONN_SYSEX))
+					continue;
 				msg->params.bristol.from = i;
 				bmidi.handle[i].callback(msg, bmidi.handle[i].param);
 				msg->params.bristol.from = hold;
@@ -185,9 +257,8 @@ bristolMidiALSARead(int dev, bristolMidiMsg *msg)
 {
 	int parsed, space;
 
-#ifdef DEBUG
-	printf("bristolMidiALSARead(%i)\n", dev);
-#endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiALSARead(%i)\n", dev);
 
 	/*
 	 * See if we can read more data from the midi device. We need to make sure
@@ -204,7 +275,6 @@ bristolMidiALSARead(int dev, bristolMidiMsg *msg)
 		 * this is the case, only partially read to the end of the buffer, then
 		 * read further into the buffer. Requires two read ops. Bummer.
 		 */
-
 		/*
 		 * Generate offset for new data, with buffer size corrections.
 		 */
@@ -245,14 +315,20 @@ bristolMidiALSARead(int dev, bristolMidiMsg *msg)
 				{
 					FD_ZERO(read_set);
 					FD_SET(bmidi.dev[dev].fd, read_set);
-#warning LINK AN OSC FD REQUEST IN HERE
+
+//#warning LINK AN OSC FD REQUEST IN HERE
 					/*
 					 * OSC will give perhaps another FD to be included. If the
 					 * request for the OSC fd > 0 put it in the list
 					 */
 
-					timeout.tv_sec = 1;
+					if (bmidi.dev[dev].flags & BRISTOL_CONN_NBLOCK)
+						timeout.tv_sec = 0;
+					else
+						timeout.tv_sec = 1;
+					timeout.tv_sec = 0;
 					timeout.tv_usec = 100000;
+					timeout.tv_usec = 10000;
 
 					if (select(bmidi.dev[dev].fd + 1, read_set,
 						NULL, NULL, &timeout) == 1)
@@ -265,6 +341,8 @@ bristolMidiALSARead(int dev, bristolMidiMsg *msg)
 					if (bmidi.dev[dev].flags & BRISTOL_CONN_NBLOCK)
 						return(BRISTOL_MIDI_DEV);
 
+					return(BRISTOL_MIDI_DEV);
+
 					printf("Midi read retry (%i)\n", getpid());
 				}
 #else
@@ -287,31 +365,35 @@ bristolMidiALSARead(int dev, bristolMidiMsg *msg)
 				/*
 				 * MARK
 				 */
-				printf("return - no data in buffer for %i\n", dev);
+				printf("no data in alsa buffer for %i (close)\n", dev);
 				msg->command = -1;
 				/*
 				 * We should consider closing whatever synth was listening
 				 * here. This would require build an EXIT message.
 				 */
-				return(BRISTOL_MIDI_DEV);
+				return(BRISTOL_MIDI_CHANNEL);
 			} else
 				count = 0;
 		}
 
 		bmidi.dev[dev].bufcount++;
-	} else
+	} else {
 		printf("Device buffer exhausted\n");
+		bmidi.dev[dev].bufindex = bmidi.dev[dev].bufcount = 0;
+		return(BRISTOL_MIDI_DEV);
+	}
 
 	/*
 	 * I really want to parse these, and then forward the messages to callback
 	 * routines registered by the bristolMidiOpen() command.
-	 */
 	if ((parsed = bristolMidiRawToMsg(&bmidi.dev[dev].buffer[0],
+	 */
+	while ((parsed = bristolMidiRawToMsg(&bmidi.dev[dev].buffer[0],
 		bmidi.dev[dev].bufcount, bmidi.dev[dev].bufindex, dev, msg)) > 0)
 	{
-		/* 
-		 * Reduce count of available space.
-		 */
+		if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+			printf("parsed %i\n", parsed);
+
 		if ((bmidi.dev[dev].bufcount -= parsed) < 0)
 		{
 			/*
@@ -342,8 +424,9 @@ bristolMidiALSARead(int dev, bristolMidiMsg *msg)
 
 		if (msg->command != 0x0ff)
 			checkcallbacks(msg);
-	} else
-		msg->command = -1;
+	}
+
+	msg->command = -1;
 
 	return(BRISTOL_MIDI_OK);
 }
