@@ -53,6 +53,21 @@ static char tmppath[BUFSZE];
 
 pthread_t lthread;
 
+void
+resetBristolCache(char *dir)
+{
+	if (dir == NULL)
+		return;
+
+	setenv("BRISTOL_CACHE", dir, 1);
+
+	if (myHome == NULL)
+		return;
+
+	free(myHome);
+	myHome = NULL;
+}
+
 /*
  * Search the file path given below to look for the desired file. Return the
  * base to that file: the bristol base irrespective of where we find the file
